import doctest

from insights.combiners import os_release
from insights.combiners.os_release import OSRelease
from insights.parsers.dmesg import DmesgLineList
from insights.parsers.installed_rpms import InstalledRpms
from insights.parsers.os_release import OsRelease
from insights.parsers.redhat_release import RedhatRelease
from insights.parsers.uname import Uname
from insights.tests import context_wrap

UNAME_86 = "Linux vm-123 4.18.0-372.19.1.el8_6.x86_64 #1 SMP Mon Jul 18 11:14:02 EDT 2022 x86_64 x86_64 x86_64 GNU/Linux"
UNAME_91 = "Linux vm-123 5.14.0-162.6.1.el9_1.x86_64 #1 SMP PREEMPT_DYNAMIC Fri Sep 30 07:36:03 EDT 2022 x86_64 x86_64 x86_64 GNU/Linux"
UNAME_ORACLE = "Linux atlnfs4testd 4.18.0-372.19.1.el8_6uek.x86_64 #1 SMP Thu Nov 7 17:01:44 PST 2013 x86_64 x86_64 x86_64 GNU/Linux"
UNAME_FEDORA = "Linux sironote.home.local 3.17.8-200.fc20.x86_64 #1 SMP Thu Jan 8 23:26:57 UTC 2015 x86_64 x86_64 x86_64 GNU/Linux"
UNAME_UNKNOWN = "Linux eslinb24.emea.nsn-net.net 2.6.39.4-9.NSN.kiuas #1 SMP Thu Feb 13 08:58:31 EET 2014 x86_64 x86_64 x86_64 GNU/Linux"

RPMS_JSON_91_WO_KERNEL = '''
{"name":"audit-libs", "epoch":"(none)", "version":"11", "release":"13.el9", "arch":"noarch", "vendor":"Red Hat, Inc.", "sigpgp":"RSA/SHA256, Sat Nov 20 18:50:43 2021, Key ID 199e2f91fd431d51"}
{"name":"basesystem", "epoch":"(none)", "version":"11", "release":"13.el9", "arch":"noarch", "vendor":"Red Hat, Inc.", "sigpgp":"RSA/SHA256, Sat Nov 20 18:50:43 2021, Key ID 199e2f91fd431d51"}
{"name":"bash", "epoch":"(none)", "version":"5.1.8", "release":"5.el9", "arch":"x86_64", "vendor":"Red Hat, Inc.", "sigpgp":"RSA/SHA256, Thu Aug 25 21:46:10 2022, Key ID 199e2f91fd431d51"}
{"name":"coreutils", "epoch":"(none)", "version":"8.32", "release":"32.el9", "arch":"x86_64", "vendor":"Red Hat, Inc.", "sigpgp":"RSA/SHA256, Thu Jun 16 12:19:44 2022, Key ID 199e2f91fd431d51"}
{"name":"dbus", "epoch":"1", "version":"1.12.20", "release":"6.el9", "arch":"x86_64", "vendor":"Red Hat, Inc.", "sigpgp":"RSA/SHA256, Tue Aug 23 23:01:36 2022, Key ID 199e2f91fd431d51"}
{"name":"dmidecode", "epoch":"1", "version":"3.3", "release":"7.el9", "arch":"x86_64", "vendor":"Red Hat, Inc.", "sigpgp":"RSA/SHA256, Tue Mar 15 15:41:19 2022, Key ID 199e2f91fd431d51"}
{"name":"dnf", "epoch":"(none)", "version":"057", "release":"13.git20220816.el9", "arch":"x86_64", "vendor":"Red Hat, Inc.", "sigpgp":"RSA/SHA256, Wed Aug 17 08:06:56 2022, Key ID 199e2f91fd431d51"}
{"name":"dracut", "epoch":"(none)", "version":"057", "release":"13.git20220816.el9", "arch":"x86_64", "vendor":"Red Hat, Inc.", "sigpgp":"RSA/SHA256, Wed Aug 17 08:06:56 2022, Key ID 199e2f91fd431d51"}
{"name":"filesystem", "epoch":"(none)", "version":"3.16", "release":"2.el9", "arch":"x86_64", "vendor":"Red Hat, Inc.", "sigpgp":"RSA/SHA256, Sat Nov 20 19:51:43 2021, Key ID 199e2f91fd431d51"}
{"name":"firewalld", "epoch":"(none)", "version":"1.1.1", "release":"3.el9", "arch":"noarch", "vendor":"Red Hat, Inc.", "sigpgp":"RSA/SHA256, Mon Aug  8 21:31:26 2022, Key ID 199e2f91fd431d51"}
{"name":"glibc", "epoch":"(none)", "version":"2.34", "release":"40.el9", "arch":"x86_64", "vendor":"Red Hat, Inc.", "sigpgp":"RSA/SHA256, Thu Jul 28 15:15:24 2022, Key ID 199e2f91fd431d51"}
{"name":"gmp", "epoch":"1", "version":"6.2.0", "release":"10.el9", "arch":"x86_64", "vendor":"Red Hat, Inc.", "sigpgp":"RSA/SHA256, Sun Nov 21 16:04:10 2021, Key ID 199e2f91fd431d51"}
{"name":"libacl", "epoch":"(none)", "version":"2.3.1", "release":"3.el9", "arch":"x86_64", "vendor":"Red Hat, Inc.", "sigpgp":"RSA/SHA256, Sun Nov 21 08:14:00 2021, Key ID 199e2f91fd431d51"}
{"name":"libgcc", "epoch":"(none)", "version":"11.3.1", "release":"2.1.el9", "arch":"x86_64", "vendor":"Red Hat, Inc.", "sigpgp":"RSA/SHA256, Thu Jul 14 22:56:25 2022, Key ID 199e2f91fd431d51"}
{"name":"libselinux", "epoch":"(none)", "version":"3.4", "release":"3.el9", "arch":"x86_64", "vendor":"Red Hat, Inc.", "sigpgp":"RSA/SHA256, Tue Jul 26 15:17:57 2022, Key ID 199e2f91fd431d51"}
{"name":"systemd", "epoch":"(none)", "version":"250", "release":"12.el9_1", "arch":"x86_64", "vendor":"Red Hat, Inc.", "sigpgp":"RSA/SHA256, Thu 29 Sep 2022 05:02:47 PM CST, Key ID 199e2f91fd431d51"}
'''.strip()
RPMS_JSON_91_W_KERNEL = RPMS_JSON_91_WO_KERNEL + """
{"name":"passwd", "epoch":"(none)", "version":"3.4", "release":"4.el9", "arch":"x86_64", "vendor":"Red Hat, Inc.", "sigpgp":"RSA/SHA256, Tue Sep 13 16:46:26 2022, Key ID 199e2f91fd431d51"}
{"name":"kernel", "epoch":"(none)", "version":"5.14.0", "release":"70.13.1.el9_0", "arch":"x86_64", "vendor":"RH, Inc.", "sigpgp":"RSA/SHA256, Mon 05 Sep 2022 09:55:09 PM CST, Key ID 199e2f91fd431d51"}
{"name":"kernel", "epoch":"(none)", "version":"5.14.0", "release":"162.6.1.el9_1", "arch":"x86_64", "vendor":"Red Hat, Inc.", "sigpgp":"RSA/SHA256, Mon 03 Oct 2022 04:18:36 PM CST, Key ID 199e2f91fd431d51"}"""
RPMS_JSON_8_NG = '''
{"name":"kernel", "epoch":"(none)", "version":"4.18.0", "release":"425.3.1.el8", "arch":"x86_64", "vendor":"RH, Inc.", "sigpgp":"RSA/SHA256, Tue Nov  8 18:10:54 2022, Key ID 99e2f91fd431d51"}
{"name":"kernel", "epoch":"(none)", "version":"4.18.0", "release":"372.19.1.el8_6uek", "arch":"x86_64", "vendor":"Oracle America", "sigpgp":"RSA/SHA256, Wed Sep 15 17:11:22 2021, Key ID 99e2f91fd431d51"}
{"name":"kernel", "epoch":"(none)", "version":"4.18.0", "release":"372.19.1.el8_6", "arch":"x86_64", "vendor":"SUSE, Inc.", "sigpgp":"RSA/SHA256, Wed Sep 15 17:11:22 2021, Key ID 99e2f91fd431d51"}
{"name":"libselinux", "epoch":"(none)", "version":"2.9", "release":"6.el8", "arch":"i686", "vendor":"RH, Inc.", "sigpgp":"RSA/SHA256, Mon 15 Aug 2022 08:55:09 PM CST, Key ID 99e2f91fd431d51"}
{"name":"dbus", "epoch":"1", "version":"1.12.8", "release":"23.el8", "arch":"x86_64", "vendor":"Red, Inc.", "sigpgp":"RSA/SHA256, Wed 07 Sep 2022 04:08:12 AM CST, Key ID 99e2f91fd431d51"}
{"name":"dracut", "epoch":"(none)", "version":"049", "release":"209.git20220815.el8", "arch":"x86_64", "vendor":"RH, Inc.", "sigpgp":"RSA/SHA256, Mon 15 Aug 2022 09:56:58 PM CST, Key ID 99e2f91fd431d51"}
{"name":"libgcc", "epoch":"(none)", "version":"8.5.0", "release":"15.el8", "arch":"x86_64", "vendor":"RH, Inc.", "sigpgp":"RSA/SHA256, Thu 21 Jul 2022 05:36:25 PM CST, Key ID 99e2f91fd431d51"}
{"name":"policycoreutils", "epoch":"(none)", "version":"2.9", "release":"20.el8", "arch":"x86_64", "vendor":"Red Hat, Inc.", "sigpgp":"RSA/SHA256, Mon 15 Aug 2022 08:51:06 PM CST, Key ID 199e2f91fd431d51"}
{"name":"glibc", "epoch":"(none)", "version":"2.28", "release":"211.el8", "arch":"x86_64", "vendor":"RH, Inc.", "sigpgp":"RSA/SHA256, Mon 29 Aug 2022 04:13:20 PM CST, Key ID 99e2f91fd431d51"}
{"name":"libacl", "epoch":"(none)", "version":"2.2.53", "release":"1.el8", "arch":"x86_64", "vendor":"Red Hat, Inc.", "sigpgp":"RSA/SHA256, Sat 15 Dec 2018 05:44:36 AM CST, Key ID 199e2f91fd431d51"}
{"name":"glibc", "epoch":"(none)", "version":"2.28", "release":"211.el8", "arch":"i686", "vendor":"RH, Inc.", "sigpgp":"RSA/SHA256, Mon 29 Aug 2022 04:12:26 PM CST, Key ID 99e2f91fd431d51"}
{"name":"libgcc", "epoch":"(none)", "version":"8.5.0", "release":"15.el8", "arch":"i686", "vendor":"Red Hat, Inc.", "sigpgp":"RSA/SHA256, Thu 21 Jul 2022 05:36:01 PM CST, Key ID 199e2f91fd431d51"}
{"name":"bash", "epoch":"(none)", "version":"4.4.20", "release":"4.el8_6", "arch":"x86_64", "vendor":"RHat, Inc.", "sigpgp":"RSA/SHA256, Mon 20 Jun 2022 09:20:51 PM CST, Key ID 99e2f91fd431d51"}
{"name":"libselinux", "epoch":"(none)", "version":"2.9", "release":"6.el8", "arch":"x86_64", "vendor":"Hat, Inc.", "sigpgp":"RSA/SHA256, Mon 15 Aug 2022 08:55:11 PM CST, Key ID 09e2f91fd431d51"}
{"name":"coreutils", "epoch":"(none)", "version":"8.30", "release":"13.el8", "arch":"x86_64", "vendor":"Hat, Inc.", "sigpgp":"RSA/SHA256, Thu 16 Jun 2022 12:18:02 PM CST, Key ID 09e2f91fd431d51"}
{"name":"firewalld", "epoch":"(none)", "version":"0.9.3", "release":"13.el8", "arch":"noarch", "vendor":"Red, Inc.", "sigpgp":"RSA/SHA256, Fri 25 Feb 2022 09:40:17 PM CST, Key ID 09e2f91fd431d51"}
{"name":"filesystem", "epoch":"(none)", "version":"3.8", "release":"6.el8", "arch":"x86_64", "vendor":"RH, Inc.", "sigpgp":"RSA/SHA256, Mon 21 Jun 2021 07:17:43 PM CST, Key ID 99e2f91fd431d51"}
{"name":"gmp", "epoch":"1", "version":"6.1.2", "release":"10.el8", "arch":"x86_64", "vendor":"RH, Inc.", "sigpgp":"RSA/SHA256, Fri 14 Jun 2019 04:58:39 PM CST, Key ID 99e2f91fd431d51"}
{"name":"basesystem", "epoch":"(none)", "version":"11", "release":"5.el8", "arch":"noarch", "vendor":"RH, Inc.", "sigpgp":"RSA/SHA256, Sat 15 Dec 2018 05:49:21 AM CST, Key ID 09e2f91fd431d51"}
{"name":"dmidecode", "epoch":"1", "version":"3.3", "release":"4.el8", "arch":"x86_64", "vendor":"Red, Inc.", "sigpgp":"RSA/SHA256, Mon 14 Mar 2022 02:13:06 PM CST, Key ID 99e2f91fd431d51"}
'''.strip()
RPMS_JSON_9_NG_RH_KERNEL = RPMS_JSON_8_NG + """
{"name":"kernel", "epoch":"(none)", "version":"5.14.0", "release":"162.6.1.el9_1", "arch":"x86_64", "vendor":"Red Hat, Inc.", "sigpgp":"RSA/SHA256, Mon 03 Oct 2022 04:18:36 PM CST, Key ID 199e2f91fd431d51"}"""
RPMS_JSON_ROCKY = '''
{"name":"rocky-release", "epoch":"1", "version":"3.3", "release":"4.el8", "arch":"x86_64", "vendor":"Red Hat, Inc.", "sigpgp":"RSA/SHA256, Mon 14 Mar 2022 02:13:06 PM CST, Key ID 99e2f91fd431d51"}
'''.strip()

DMESG_ORACLE = """
Linux version kernel-4.18.0-372.19.1.el8_6uek.x86_64 (mockbuild@ca-build56.us.oracle.com) (gcc version 4.1.2 20080704 (Red Hat 4.1.2-54)) #1 SMP Mon Sep 30 16:46:32 PDT 2013
""".strip()
DMESG_CENTOS = """
[    0.000000] Linux version 4.18.0-240.el8.x86_64 (mockbuild@kbuilder.bsys.centos.org) (gcc version 8.4.1 20200928 (Red Hat 8.4.1-1) (GCC)) #1 SMP Tue Apr 13 16:24:22 UTC 2021
""".strip()
DMESG_SUSE = """
Linux version 2.6.32-431.23.3.el6.x86_64 (sandman@ceph01t6) (gcc version 4.4.7 20120313 (Novell 4.4.7-4) (GCC) ) #1 SMP Tue Jul 29 17:05:14 EDT 2014
""".strip()
DMESG_UNKNOWN = """
Linux version 2.6.32-431.17.1.el6.x86_64 (mockbuild@lxdist01) (gcc version 4.4.7 20120313 (Red Hat 4.4.7-4) (GCC) ) #1 SMP Thu May 8 08:33:50 CEST 2014
""".strip()
DMESG_NG = """
Initializing cgroup subsys cpu
Command line: ro root=/dev/vg00/lvol1
""".strip()
DMESG_REDHAT = """
[    0.000000] Linux version 5.14.0-162.6.1.el9_1.x86_64 (mockbuild@x86-vm-07.build.eng.bos.redhat.com) (gcc (GCC) 11.3.1 20220421 (Red Hat 11.3.1-2), GNU ld version 2.35.2-24.el9) #1 SMP PREEMPT_DYNAMIC Tue Dec 20 06:06:30 EST 2022
""".strip()

REDHAT_RELEASE_86 = "Red Hat Enterprise Linux release 8.6 (Ootpa)"
REDHAT_RELEASE_FEDORA = "Fedora release 23 (Twenty Three)"
REDHAT_RELEASE_UNKNOWN = "Test OS"

OS_RELEASE_RH = """
NAME="Red Hat Enterprise Linux"
ID="rhel"
""".strip()
OS_RELEASE_OL = """
NAME="Oracle Linux Server"
ID="ol"
PRETTY_NAME="Red Hat Enterprise Linux"
""".strip()
OS_RELEASE_CENTOS = """
NAME="CentOS Stream"
ID="centos"
PRETTY_NAME="CentOS Stream 9"
""".strip()
OS_RELEASE_UNKNOWN = """
NAME="Test OS"
ID="test"
PRETTY_NAME="Test OS"
""".strip()

OS_RELEASE_RHEL_AI = """
NAME="Red Hat Enterprise Linux"
VERSION="9.20240630.0.4 (Plow)"
ID="rhel"
ID_LIKE="fedora"
VERSION_ID="9.4"
PLATFORM_ID="platform:el9"
PRETTY_NAME="Red Hat Enterprise Linux 9.20240630.0.4 (Plow)"
ANSI_COLOR="0;31"
LOGO="fedora-logo-icon"
CPE_NAME="cpe:/o:redhat:enterprise_linux:9::baseos"
HOME_URL="https://www.redhat.com/"
DOCUMENTATION_URL="https://access.redhat.com/documentation/en-us/red_hat_enterprise_linux/9"
BUG_REPORT_URL="https://bugzilla.redhat.com/"
REDHAT_BUGZILLA_PRODUCT="Red Hat Enterprise Linux 9"
REDHAT_BUGZILLA_PRODUCT_VERSION=9.4
REDHAT_SUPPORT_PRODUCT="Red Hat Enterprise Linux"
REDHAT_SUPPORT_PRODUCT_VERSION="9.4"
OSTREE_VERSION='9.20240630.0'
VARIANT_ID=rhel_ai
VARIANT="RHEL AI"
BUILD_ID='v1.1.3'
""".strip()


def test_is_rhel():
    # RHEL, uname only
    uname = Uname(context_wrap(UNAME_91))
    result = OSRelease(uname, None, None, None, None)
    assert result.is_rhel is True
    assert result.release == "RHEL"
    assert result.reasons == {}
    assert result.name == "RHEL"  # the same to self.release when '/etc/os-release' is unavailable
    assert result.is_rhel_compatible is False

    # RHEL, dmesg only
    dmesg = DmesgLineList(context_wrap(DMESG_REDHAT))
    result = OSRelease(None, dmesg, None, None, None)
    assert result.is_rhel is True
    assert result.release == "RHEL"
    assert result.product == "RHEL"
    assert result.reasons == {}
    assert result.name == "RHEL"  # the same to self.release when '/etc/os-release' is unavailable
    assert result.is_rhel_compatible is False

    # RHEL, dmesg and uname
    dmesg = DmesgLineList(context_wrap(DMESG_REDHAT))
    result = OSRelease(uname, dmesg, None, None, None)
    assert result.is_rhel is True
    assert result.release == "RHEL"
    assert result.reasons == {}
    assert result.name == "RHEL"  # the same to self.release when '/etc/os-release' is unavailable
    assert result.is_rhel_compatible is False

    # RHEL, rpms only
    rpms = InstalledRpms(context_wrap(RPMS_JSON_91_WO_KERNEL))
    result = OSRelease(None, None, rpms, None, None)
    assert result.is_rhel is True
    assert result.release == "RHEL"
    assert result.reasons == {}
    assert result.name == "RHEL"  # the same to self.release when '/etc/os-release' is unavailable
    assert result.is_rhel_compatible is False

    rpms = InstalledRpms(context_wrap(RPMS_JSON_91_W_KERNEL))
    result = OSRelease(None, None, rpms, None, None)
    assert result.is_rhel is True
    assert result.release == "RHEL"
    assert result.reasons == {}
    assert result.name == "RHEL"  # the same to self.release when '/etc/os-release' is unavailable
    assert result.is_rhel_compatible is False

    # RHEL, rpms and uname
    rpms = InstalledRpms(context_wrap(RPMS_JSON_91_W_KERNEL))
    result = OSRelease(uname, None, rpms, None, None)
    assert result.is_rhel is True
    assert result.release == "RHEL"
    assert result.reasons == {}
    assert result.name == "RHEL"  # the same to self.release when '/etc/os-release' is unavailable
    assert result.is_rhel_compatible is False

    # RHEL, rpms, dmesg and uname
    rpms = InstalledRpms(context_wrap(RPMS_JSON_91_W_KERNEL))
    result = OSRelease(uname, dmesg, rpms, None, None)
    assert result.is_rhel is True
    assert result.release == "RHEL"
    assert result.reasons == {}
    assert result.name == "RHEL"  # the same to self.release when '/etc/os-release' is unavailable
    assert result.is_rhel_compatible is False

    # RHEL, os-release
    osr = OsRelease(context_wrap(OS_RELEASE_RH))
    result = OSRelease(None, None, None, osr, None)
    assert result.is_rhel is True
    assert result.release == "RHEL"
    assert result.reasons == {}
    assert result.name == "Red Hat Enterprise Linux"
    assert result.is_rhel_compatible is False

    # RHEL, os-release
    osr = OsRelease(context_wrap(OS_RELEASE_RHEL_AI))
    result = OSRelease(None, None, None, osr, None)
    assert result.is_rhel is True
    assert result.release == "RHEL"
    assert result.reasons == {}
    assert result.name == "Red Hat Enterprise Linux"
    assert result.is_rhel_compatible is False

    # RHEL, redhat-release
    rhr = RedhatRelease(context_wrap(REDHAT_RELEASE_86))
    result = OSRelease(None, None, None, None, rhr)
    assert result.is_rhel is True
    assert result.release == "RHEL"
    assert result.reasons == {}
    assert result.name == "RHEL"  # the same to self.release when '/etc/os-release' is unavailable
    assert result.is_rhel_compatible is False

    # RHEL, Uname is unknown, rpms is OK
    uname = Uname(context_wrap(UNAME_UNKNOWN))
    rpms = InstalledRpms(context_wrap(RPMS_JSON_91_W_KERNEL))
    result = OSRelease(uname, None, rpms, None, None)
    assert result.is_rhel is True
    assert result.release == "RHEL"
    assert result.name == "RHEL"  # the same to self.release when '/etc/os-release' is unavailable
    assert result.is_rhel_compatible is False

    # RHEL: Unknown rpms, too many faulty, but running kernel is from Red Hat
    rpms = InstalledRpms(context_wrap(RPMS_JSON_9_NG_RH_KERNEL))
    uname = Uname(context_wrap(UNAME_91))
    result = OSRelease(uname, None, rpms, None, None)
    assert result.is_rhel is True
    assert result.release == "RHEL"
    assert result.name == "RHEL"  # the same to self.release when '/etc/os-release' is unavailable
    assert result.is_rhel_compatible is False


def test_not_rhel():
    # NON-RHEL: Nothing
    result = OSRelease(None, None, None, None, None)
    assert result.is_rhel is False
    assert result.release == "Unknown"
    assert result.reasons.get('reason') == "Nothing available to check"
    assert result.name == "Unknown"  # the same to self.release when '/etc/os-release' is unavailable
    assert result.is_rhel_compatible is False

    # NON-RHEL: Rocky rpms
    rpms = InstalledRpms(context_wrap(RPMS_JSON_ROCKY))
    result = OSRelease(None, None, rpms, None, None)
    assert result.is_rhel is False
    assert result.release == "Rocky"
    assert result.reasons['release'] == 'rocky-release-3.3-4.el8'
    assert result.name == "Rocky"  # the same to self.release when '/etc/os-release' is unavailable
    assert result.is_rhel_compatible is False

    # NON-RHEL: Unknown rpms: too many faulty pkgs
    rpms = InstalledRpms(context_wrap(RPMS_JSON_8_NG))
    result = OSRelease(None, None, rpms, None, None)
    assert result.is_rhel is False
    assert result.release == "Unknown"
    assert result.reasons['faulty_packages'] == [
        'basesystem-11-5.el8', 'bash-4.4.20-4.el8_6',
        'coreutils-8.30-13.el8', 'dbus-1.12.8-23.el8',
        'dmidecode-3.3-4.el8', 'dracut-049-209.git20220815.el8',
        'filesystem-3.8-6.el8', 'firewalld-0.9.3-13.el8',
        'glibc-2.28-211.el8', 'gmp-6.1.2-10.el8',
        'libgcc-8.5.0-15.el8', 'libselinux-2.9-6.el8']
    assert result.name == "Unknown"  # the same to self.release when '/etc/os-release' is unavailable
    assert result.is_rhel_compatible is False

    # NON-RHEL: SUSE rpms, too many faulty
    uname = Uname(context_wrap(UNAME_86))
    rpms = InstalledRpms(context_wrap(RPMS_JSON_8_NG))
    result = OSRelease(uname, None, rpms, None, None)
    assert result.is_rhel is False
    assert result.release == "SUSE"  # From RPMs, Kenrel is from SUSE per Uname
    assert result.reasons['faulty_packages'] == [
        'basesystem-11-5.el8', 'bash-4.4.20-4.el8_6',
        'coreutils-8.30-13.el8', 'dbus-1.12.8-23.el8',
        'dmidecode-3.3-4.el8', 'dracut-049-209.git20220815.el8',
        'filesystem-3.8-6.el8', 'firewalld-0.9.3-13.el8',
        'glibc-2.28-211.el8', 'gmp-6.1.2-10.el8',
        'kernel-4.18.0-372.19.1.el8_6', 'libgcc-8.5.0-15.el8',
        'libselinux-2.9-6.el8']
    assert result.reasons['kernel_vendor'] == 'SUSE, Inc.'
    assert 'kernel' not in result.reasons  # No Uname
    assert 'build_info' not in result.reasons  # No Dmesg
    assert result.name == "SUSE"  # the same to self.release when '/etc/os-release' is unavailable
    assert result.is_rhel_compatible is False

    # NON-RHEL: BAD dmesg only
    dmesg = DmesgLineList(context_wrap(DMESG_NG))
    result = OSRelease(None, dmesg, None, None, None)
    assert result.is_rhel is False
    assert result.release == "Unknown"
    assert result.reasons.get('reason') == "Nothing available to check"
    assert result.name == "Unknown"  # the same to self.release when '/etc/os-release' is unavailable
    assert result.is_rhel_compatible is False

    # NON-RHEL: SUSE dmesg only
    dmesg = DmesgLineList(context_wrap(DMESG_SUSE))
    result = OSRelease(None, dmesg, None, None, None)
    assert result.is_rhel is False
    assert result.release == "SUSE"
    assert result.reasons['build_info'] == DMESG_SUSE
    assert result.name == "SUSE"  # the same to self.release when '/etc/os-release' is unavailable
    assert result.is_rhel_compatible is False

    # NON-RHEL: CentOS dmesg only
    dmesg = DmesgLineList(context_wrap(DMESG_CENTOS))
    result = OSRelease(None, dmesg, None, None, None)
    assert result.is_rhel is False
    assert result.release == "CentOS"
    assert result.reasons['build_info'] == DMESG_CENTOS
    assert result.name == "CentOS"  # the same to self.release when '/etc/os-release' is unavailable
    assert result.is_rhel_compatible is True

    # NON-RHEL: Uknown dmesg only
    dmesg = DmesgLineList(context_wrap(DMESG_UNKNOWN))
    result = OSRelease(None, dmesg, None, None, None)
    assert result.is_rhel is False
    assert result.release == "Unknown"
    assert result.reasons['build_info'] == DMESG_UNKNOWN
    assert result.name == "Unknown"  # the same to self.release when '/etc/os-release' is unavailable
    assert result.is_rhel_compatible is False

    # NON-RHEL: Oracle uname only
    uname = Uname(context_wrap(UNAME_ORACLE))
    result = OSRelease(uname, None, None, None, None)
    assert result.is_rhel is False
    assert result.release == "Oracle"
    assert result.reasons.get('kernel') == "4.18.0-372.19.1.el8_6uek.x86_64"
    assert result.name == "Oracle"  # the same to self.release when '/etc/os-release' is unavailable
    assert result.is_rhel_compatible is False

    # NON-RHEL: Fedora uname only
    uname = Uname(context_wrap(UNAME_FEDORA))
    result = OSRelease(uname, None, None, None, None)
    assert result.is_rhel is False
    assert result.release == "Fedora"
    assert result.reasons.get('kernel') == "3.17.8-200.fc20.x86_64"
    assert result.name == "Fedora"  # the same to self.release when '/etc/os-release' is unavailable
    assert result.is_rhel_compatible is False

    # NON-RHEL: Unknown uname only
    uname = Uname(context_wrap(UNAME_UNKNOWN))
    result = OSRelease(uname, None, None, None, None)
    assert result.is_rhel is False
    assert result.release == "Unknown"
    assert result.reasons.get('kernel') == "2.6.39.4-9.NSN.kiuas"
    assert result.name == "Unknown"  # the same to self.release when '/etc/os-release' is unavailable
    assert result.is_rhel_compatible is False

    # NON-RHEL: Oracle Uname + Bad Dmesg
    dmesg = DmesgLineList(context_wrap(DMESG_NG))
    uname = Uname(context_wrap(UNAME_ORACLE))
    result = OSRelease(uname, dmesg, None, None, None)
    assert result.is_rhel is False
    assert result.release == "Oracle"  # From Uname
    assert result.reasons['kernel'] == "4.18.0-372.19.1.el8_6uek.x86_64"
    assert 'kernel_vendor' not in result.reasons  # No RPMs
    assert result.name == "Oracle"  # the same to self.release when '/etc/os-release' is unavailable
    assert result.is_rhel_compatible is False

    # NON-RHEL: Oracle Uname + Unknown Dmesg
    dmesg = DmesgLineList(context_wrap(DMESG_UNKNOWN))
    uname = Uname(context_wrap(UNAME_ORACLE))
    result = OSRelease(uname, dmesg, None, None, None)
    assert result.is_rhel is False
    assert result.release == "Oracle"  # From Uname
    assert result.reasons['kernel'] == "4.18.0-372.19.1.el8_6uek.x86_64"
    assert 'kernel_vendor' not in result.reasons  # No RPMs
    assert result.name == "Oracle"  # the same to self.release when '/etc/os-release' is unavailable
    assert result.is_rhel_compatible is False

    # NON-RHEL: SUSE dmesg + unknown RPMs
    dmesg = DmesgLineList(context_wrap(DMESG_SUSE))
    rpms = InstalledRpms(context_wrap(RPMS_JSON_8_NG))
    result = OSRelease(None, dmesg, rpms, None, None)
    assert result.is_rhel is False
    assert result.release == "SUSE"  # From dmesg
    assert 'kernel' not in result.reasons  # No Uname
    assert 'kernel_vendor' not in result.reasons  # No Uname
    assert result.reasons['build_info'] == DMESG_SUSE
    assert result.name == "SUSE"  # the same to self.release when '/etc/os-release' is unavailable
    assert result.is_rhel_compatible is False

    # NON-RHEL: unknown Uname + Oracle dmesg +  unknown RPMs
    dmesg = DmesgLineList(context_wrap(DMESG_ORACLE))
    uname = Uname(context_wrap(UNAME_UNKNOWN))
    rpms = InstalledRpms(context_wrap(RPMS_JSON_8_NG))
    result = OSRelease(uname, dmesg, rpms, None, None)
    assert result.is_rhel is False
    assert result.release == "Oracle"  # From dmesg
    assert result.name == "Oracle"  # the same to self.release when '/etc/os-release' is unavailable
    assert result.reasons['build_info'] == DMESG_ORACLE
    assert result.is_rhel_compatible is False

    # NON-RHEL: unknown Uname + unknown RPMs
    uname = Uname(context_wrap(UNAME_UNKNOWN))
    rpms = InstalledRpms(context_wrap(RPMS_JSON_8_NG))
    result = OSRelease(uname, None, rpms, None, None)
    assert result.is_rhel is False
    assert result.release == "Unknown"  # From uname
    assert result.name == "Unknown"  # the same to self.release when '/etc/os-release' is unavailable
    assert result.reasons['kernel'] == "2.6.39.4-9.NSN.kiuas"
    assert result.is_rhel_compatible is False

    # NON-RHEL: unknown Uname + unknown Dmesg +  unknown RPMs
    dmesg = DmesgLineList(context_wrap(DMESG_UNKNOWN))
    uname = Uname(context_wrap(UNAME_UNKNOWN))
    rpms = InstalledRpms(context_wrap(RPMS_JSON_8_NG))
    result = OSRelease(uname, dmesg, rpms, None, None)
    assert result.is_rhel is False
    assert result.release == "Unknown"  # From dmesg
    assert result.reasons['build_info'] == DMESG_UNKNOWN
    assert result.name == "Unknown"  # the same to self.release when '/etc/os-release' is unavailable
    assert result.is_rhel_compatible is False

    # NON-RHEL: SUSE Uname + unknown Dmesg + SUSE RPMs
    dmesg = DmesgLineList(context_wrap(DMESG_UNKNOWN))
    uname = Uname(context_wrap(UNAME_86))
    rpms = InstalledRpms(context_wrap(RPMS_JSON_9_NG_RH_KERNEL))
    result = OSRelease(uname, dmesg, rpms, None, None)
    assert result.is_rhel is False
    assert result.release == "SUSE"  # From RPMs, Kernel is from SUSE per the uname
    assert sorted(result.reasons['faulty_packages']) == sorted([
        'basesystem-11-5.el8', 'bash-4.4.20-4.el8_6',
        'coreutils-8.30-13.el8', 'dbus-1.12.8-23.el8',
        'dmidecode-3.3-4.el8', 'dracut-049-209.git20220815.el8',
        'filesystem-3.8-6.el8', 'firewalld-0.9.3-13.el8',
        'glibc-2.28-211.el8', 'gmp-6.1.2-10.el8',
        'kernel-4.18.0-372.19.1.el8_6', 'libgcc-8.5.0-15.el8',
        'libselinux-2.9-6.el8'])
    assert result.reasons['kernel_vendor'] == 'SUSE, Inc.'
    assert result.name == "SUSE"  # the same to self.release when '/etc/os-release' is unavailable
    assert result.is_rhel_compatible is False

    # RHEL, os-release only
    osr = OsRelease(context_wrap(OS_RELEASE_UNKNOWN))
    result = OSRelease(None, None, None, osr, None)
    assert result.is_rhel is False
    assert result.release == "Test"
    assert result.reasons == {'reason': 'NON-RHEL: os-release'}
    assert result.name == "Test OS"
    assert result.is_rhel_compatible is False

    osr = OsRelease(context_wrap(OS_RELEASE_CENTOS))
    result = OSRelease(None, None, None, osr, None)
    assert result.is_rhel is False
    assert result.release == "CentOS"
    assert result.reasons == {'reason': 'NON-RHEL: os-release'}
    assert result.name == "CentOS Stream"
    assert result.is_rhel_compatible is True

    osr = OsRelease(context_wrap(OS_RELEASE_OL))
    result = OSRelease(None, None, None, osr, None)
    assert result.is_rhel is False
    assert result.release == "Oracle"
    assert result.reasons == {'reason': 'NON-RHEL: os-release'}
    assert result.name == "Oracle Linux Server"
    assert result.is_rhel_compatible is False

    # RHEL, redhat-release only
    rhr = RedhatRelease(context_wrap(REDHAT_RELEASE_FEDORA))
    result = OSRelease(None, None, None, None, rhr)
    assert result.is_rhel is False
    assert result.release == "Fedora"
    assert result.reasons == {'reason': 'NON-RHEL: redhat-release'}
    assert result.name == "Fedora"
    assert result.is_rhel_compatible is False

    rhr = RedhatRelease(context_wrap(REDHAT_RELEASE_UNKNOWN))
    result = OSRelease(None, None, None, None, rhr)
    assert result.is_rhel is False
    assert result.release == "Test"
    assert result.reasons == {'reason': 'NON-RHEL: redhat-release'}
    assert result.name == "Test"
    assert result.is_rhel_compatible is False

    # RHEL, Oracle os-release + RHEL redhat-release
    osr = OsRelease(context_wrap(OS_RELEASE_OL))
    rhr = RedhatRelease(context_wrap(REDHAT_RELEASE_86))
    result = OSRelease(None, None, None, osr, rhr)
    assert result.is_rhel is False
    assert result.release == "Oracle"
    assert result.reasons == {'reason': 'NON-RHEL: os-release'}
    assert result.name == "Oracle Linux Server"
    assert result.is_rhel_compatible is False

    # RHEL, RHEL os-release + Fedora redhat-release
    osr = OsRelease(context_wrap(OS_RELEASE_RH))
    rhr = RedhatRelease(context_wrap(REDHAT_RELEASE_FEDORA))
    result = OSRelease(None, None, None, osr, rhr)
    assert result.is_rhel is False
    assert result.release == "Fedora"
    assert result.reasons == {'reason': 'NON-RHEL: redhat-release'}
    assert result.name == "Red Hat Enterprise Linux"
    assert result.is_rhel_compatible is False


def test_osr_doc():
    dmesg = DmesgLineList(context_wrap(DMESG_ORACLE))
    uname = Uname(context_wrap(UNAME_ORACLE))
    rpms = InstalledRpms(context_wrap(RPMS_JSON_8_NG))
    osr = OsRelease(context_wrap(OS_RELEASE_OL))
    env = {
        'osr': OSRelease(uname, dmesg, rpms, osr, None),
    }
    failed, total = doctest.testmod(os_release, globs=env)
    assert failed == 0



"""
LsAttr - command ``lsattr <files_or_dirs>``
===========================================
"""

from insights.core import Parser
from insights.core.plugins import parser
from insights.specs import Specs
from insights.core.dr import SkipComponent


@parser(Specs.lsattr)
class LsAttr(Parser, dict):
    """
    Parses the output of the ``lsattr <files_or_dirs>`` command.
    It stores the mapping of the file and its attributes to a dict,
    the key is the file path, the value is the attributes by removing
    the useless dashes. If the file or dir does not exist, it stores
    a list of the whole error line in the specific "non-exist" key.

    Sample output::

        ---------------- ./grub2-tools-2.02-0.86.el7.x86_64.rpm
        ------i--------- ./grub2-common-2.02-0.86.el7.noarch.rpm
        ---------------- ./grub2-tools-minimal-2.02-0.86.el7.x86_64.rpm
        lsattr: No such file or directory while trying to stat a/f

    Raises:
        SkipComponent: when nothing is parsed out

    Examples:
        >>> './grub2-common-2.02-0.86.el7.noarch.rpm' in lsattr_obj
        True
        >>> lsattr_obj['./grub2-common-2.02-0.86.el7.noarch.rpm']
        'i'
        >>> 'a/f' in lsattr_obj
        False
        >>> lsattr_obj['non-exist'][0]
        'lsattr: No such file or directory while trying to stat a/f'
    """
    def parse_content(self, content):
        for line in content:
            if not line.strip():
                continue
            items = line.split(None, 1)
            if len(items) != 2:
                raise SkipComponent
            if 'no such file or directory' in line.lower():
                self.setdefault('non-exist', []).append(line.strip())
            else:
                attr = items[0].strip().replace('-', '')
                # still store the file when the attribute is empty, in case
                # a direcotry is checked and we can not know if the file is not
                # in the dir, or the attribute is empty if we don't store it.
                self[items[1].strip()] = attr
        if not self:
            raise SkipComponent



import pytest

from insights.combiners.ps import Ps
from insights.core import dr, filters
from insights.core.context import HostContext
from insights.core.exceptions import SkipComponent
from insights.core.spec_factory import DatasourceProvider
from insights.parsers.ps import PsEoCmd
from insights.specs import Specs
from insights.specs.datasources.package_provides import cmd_and_pkg, get_package
from insights.tests import context_wrap

JAVA_PATH_1 = '/usr/bin/java'
JAVA_PATH_2 = '/usr/lib/jvm/java-1.8.0-openjdk-1.8.0.292.b10-1.el7_9.x86_64/jre/bin/java'
JAVA_PATH_BAD = '/random/java'
JAVA_PATH_ERR = '/error/java'
JAVA_PKG_2 = 'java-1.8.0-openjdk-headless-1.8.0.292.b10-1.el7_9.x86_64'
HTTPD_PATH = '/usr/sbin/httpd'
HTTPD_PKG = 'httpd-2.4.6-97.el7_9.x86_64'


class FakeContext(HostContext):
    def shell_out(self, cmd, split=True, timeout=None, keep_rc=False, env=None, signum=None):
        tmp_cmd = cmd.strip().split()
        shell_cmd = tmp_cmd[0]
        arg = tmp_cmd[-1]
        if 'readlink' in shell_cmd:
            if arg == JAVA_PATH_1:
                return (0, [JAVA_PATH_2, ])
            elif arg == JAVA_PATH_ERR:
                return (1, ['file not found', ])
            elif arg.startswith('/'):
                return (0, [arg, ])
        elif 'rpm' in shell_cmd:
            if arg == JAVA_PATH_2:
                return (0, [JAVA_PKG_2, ])
            elif arg == HTTPD_PATH:
                return (0, [HTTPD_PKG, ])
            else:
                return (1, ['file {0} is not owned by any package'.format(arg), ])
        elif 'which' in shell_cmd:
            if 'exception' in arg:
                raise Exception()
            elif arg.startswith('/'):
                return [tmp_cmd[-1], ]
            elif arg.endswith('java'):
                return ['/usr/bin/java', ]
            elif arg.endswith('httpd'):
                return ['/usr/sbin/httpd', ]

        raise Exception()


def setup_function(func):
    if Specs.package_provides_command in filters._CACHE:
        del filters._CACHE[Specs.package_provides_command]
    if Specs.package_provides_command in filters.FILTERS:
        del filters.FILTERS[Specs.package_provides_command]

    if func is test_cmd_and_pkg:
        filters.add_filter(Specs.package_provides_command, ['httpd', 'java'])
    elif func is test_cmd_and_pkg_not_found:
        filters.add_filter(Specs.package_provides_command, ['not_found'])


def teardown_function(func):
    if func is test_cmd_and_pkg or func is test_cmd_and_pkg_not_found:
        del filters.FILTERS[Specs.package_provides_command]


def test_get_package():
    ctx = FakeContext()

    result = get_package(ctx, '/usr/bin/java')
    assert result == 'java-1.8.0-openjdk-headless-1.8.0.292.b10-1.el7_9.x86_64'

    result = get_package(ctx, '/usr/lib/jvm/java-1.8.0-openjdk-1.8.0.292.b10-1.el7_9.x86_64/jre/bin/java')
    assert result == 'java-1.8.0-openjdk-headless-1.8.0.292.b10-1.el7_9.x86_64'


def test_get_package_bad():
    ctx = FakeContext()

    result = get_package(ctx, JAVA_PATH_BAD)
    assert result is None


def test_get_package_err():
    ctx = FakeContext()

    result = get_package(ctx, JAVA_PATH_ERR)
    assert result is None


PS_EO_CMD = """
   PID  PPID NLWP COMMAND
     1     0    1 /usr/lib/systemd/systemd --switched-root --system --deserialize 22
     2     0    1 [kthreadd]
   988     2    1 /usr/sbin/httpd -DFOREGROUND
  1036     2    1 /usr/sbin/httpd -DFOREGROUND
  1037     2    1 /usr/sbin/httpd -DFOREGROUND
  1038     2    1 /usr/sbin/httpd -DFOREGROUND
  1039     2    1 /usr/sbin/httpd -DFOREGROUND
  1040     2    1 /usr/local/sbin/httpd -DFOREGROUND
 28218     2    1 /usr/bin/java TestSleepMethod1
 28219     2    1 java TestSleepMethod1
 28240     2    1 /usr/lib/jvm/java-1.8.0-openjdk-1.8.0.292.b10-1.el7_9.x86_64/jre/bin/java TestSleepMethod2
333083     2    1 /home/user3/apps/pycharm-2021.1.1/jbr/bin/java -classpath /home/user3/apps/pycharm-2021.1.1/lib/bootstrap.jar:/home/user3/apps/pycharm-2021.1.1/lib/util.jar:/home/user3/apps/pycharm-2021.1.1/lib/jdom.jar:/home/user3/apps/pycharm-2021.1.1/lib/log4j.jar:/home/user3/apps/pycharm-2021.1.1/lib/jna.jar -Xms128m -Xmx2048m -XX:ReservedCodeCacheSize=512m -XX:+UseG1GC -XX:SoftRefLRUPolicyMSPerMB=50 -XX:CICompilerCount=2 -XX:+HeapDumpOnOutOfMemoryError -XX:-OmitStackTraceInFastThrow -ea -Dsun.io.useCanonCaches=false -Djdk.http.auth.tunneling.disabledSchemes="" -Djdk.attach.allowAttachSelf=true -Djdk.module.illegalAccess.silent=true -Dkotlinx.coroutines.debug=off -Dsun.tools.attach.tmp.only=true -XX:ErrorFile=/home/user3/java_error_in_pycharm_%p.log -XX:HeapDumpPath=/home/user3/java_error_in_pycharm_.hprof -Didea.vendor.name=JetBrains -Didea.paths.selector=PyCharm2021.1 -Djb.vmOptionsFile=/home/user3/.config/JetBrains/PyCharm2021.1/pycharm64.vmoptions -Didea.platform.prefix=Python com.intellij.idea.Main
"""

EXPECTED = DatasourceProvider(
    "\n".join([
        "{0} {1}".format(HTTPD_PATH, HTTPD_PKG),
        "{0} {1}".format(JAVA_PATH_1, JAVA_PKG_2),
        "{0} {1}".format(JAVA_PATH_2, JAVA_PKG_2)
    ]),
    relative_path='insights_commands/package_provides_command'
)


def test_cmd_and_pkg():
    pseo = PsEoCmd(context_wrap(PS_EO_CMD))
    ps = Ps(None, None, None, None, None, pseo)
    broker = dr.Broker()
    broker[HostContext] = FakeContext()
    broker[Ps] = ps

    result = cmd_and_pkg(broker)
    assert result is not None
    assert sorted(result.content) == sorted(EXPECTED.content)


def test_cmd_and_pkg_no_filters():
    pseo = PsEoCmd(context_wrap(PS_EO_CMD))
    ps = Ps(None, None, None, None, None, pseo)
    broker = dr.Broker()
    broker[HostContext] = FakeContext()
    broker[Ps] = ps

    with pytest.raises(SkipComponent):
        cmd_and_pkg(broker)


def test_cmd_and_pkg_not_found():
    pseo = PsEoCmd(context_wrap(PS_EO_CMD))
    ps = Ps(None, None, None, None, None, pseo)
    broker = dr.Broker()
    broker[HostContext] = FakeContext()
    broker[Ps] = ps

    with pytest.raises(SkipComponent):
        cmd_and_pkg(broker)



"""
Custom datasource relevant mount command
"""
from insights.core.context import HostContext
from insights.core.exceptions import SkipComponent
from insights.core.plugins import datasource
from insights.parsers.mount import ProcMounts


@datasource(ProcMounts, HostContext)
def xfs_mounts(broker):
    """
    Return the required mount points with 'xfs' type.

    Raises:
        SkipComponent: Raised if no 'xfs' mount points is available

    Returns:
        List[str]: Sorted list of 'xfs' mount points.
    """
    xfs_mnt = broker[ProcMounts].search(mount_type='xfs')
    if xfs_mnt:
        return sorted(m.mount_point for m in xfs_mnt)
    raise SkipComponent


@datasource(ProcMounts)
def dumpdev_list(broker):
    mnt = broker[ProcMounts]
    mounted_dev = [m.mounted_device for m in mnt if m.mount_type in ('ext2', 'ext3', 'ext4')]
    if mounted_dev:
        return sorted(mounted_dev)
    raise SkipComponent



from pytest import mark

from insights.client.config import InsightsConfig
from insights.core.spec_cleaner import Cleaner

test_data = 'test\nabcd\n \n\n1234\npwd: p4ssw0rd\n'.splitlines()
test_data_sensitive = 'test \n\n\nabcd\n1234\npassword: p4ssw0rd\n'.splitlines()


@mark.parametrize("obfuscate", [True, False])
def test_redact_line_changed_password(obfuscate):
    conf = InsightsConfig(obfuscate=obfuscate)

    pp = Cleaner(conf, {})
    ret = pp.clean_content(test_data_sensitive, [])
    # content IS changed
    assert test_data != ret
    assert 'p4ssw0rd' not in ret[-1]
    assert '********' in ret[-1]


@mark.parametrize("obfuscate", [True, False])
def test_redact_keyword_empty_not_change(obfuscate):
    conf = InsightsConfig(obfuscate=obfuscate)

    pp = Cleaner(conf, {})  # empty keywords
    ret = pp.clean_content(test_data, [])
    # content is NOT changed
    assert test_data == ret


@mark.parametrize("obfuscate", [True, False])
def test_redact_keyword_changed_keyword(obfuscate):
    conf = InsightsConfig(obfuscate=obfuscate)

    pp = Cleaner(conf, {'keywords': ['test']})
    ret = pp.clean_content(test_data, [])
    # content IS changed
    assert test_data != ret
    assert 'test' not in ret[0]
    assert 'keyword0' in ret[0]
    assert ret[1] == test_data[1]


@mark.parametrize("obfuscate", [True, False])
def test_redact_keyword_no_such_keyword_to_change(obfuscate):
    conf = InsightsConfig(obfuscate=obfuscate)

    pp = Cleaner(conf, {'keywords': ['t_e_s_t']})  # no such keyword
    ret = pp.clean_content(test_data, [])
    # content is NOT changed
    assert test_data == ret


@mark.parametrize("obfuscate", [True, False])
def test_redact_keyword_disabled_by_no_redact(obfuscate):
    conf = InsightsConfig(obfuscate=obfuscate)

    pp = Cleaner(conf, {'keywords': 'test'})
    ret = pp.clean_content(test_data, [], no_redact=True)
    # content is NOT changed
    assert test_data == ret


@mark.parametrize("obfuscate", [True, False])
def test_redact_patterns_exclude_regex(obfuscate):
    conf = InsightsConfig(obfuscate=obfuscate)

    rm_conf = {'patterns': {'regex': ['12.*4', '^abcd']}}
    pp = Cleaner(conf, rm_conf)
    ret = pp.clean_content(test_data, [])
    # content IS changed
    assert test_data != ret
    assert '1234' not in ret
    assert 'abcd' not in ret


@mark.parametrize("obfuscate", [True, False])
def test_redact_result_empty(obfuscate):
    conf = InsightsConfig(obfuscate=obfuscate)

    rm_conf = {'patterns': {'regex': [' ', '12.*4', '^abcd', 'test', 'pwd', 'w0rd']}}
    pp = Cleaner(conf, rm_conf)
    ret = pp.clean_content(test_data, [])
    # result content is Empty
    assert len(ret) == 0


@mark.parametrize("obfuscate", [True, False])
def test_redact_patterns_exclude_no_regex(obfuscate):
    conf = InsightsConfig(obfuscate=obfuscate)

    rm_conf = {'patterns': ['1234', 'abcde']}
    pp = Cleaner(conf, rm_conf)
    ret = pp.clean_content(test_data, [])
    # content IS changed
    assert test_data != ret
    assert '1234' not in ret
    assert 'abcd' in ret


@mark.parametrize("obfuscate", [True, False])
def test_redact_patterns_exclude_empty(obfuscate):
    conf = InsightsConfig(obfuscate=obfuscate)

    rm_conf = {'patterns': []}
    pp = Cleaner(conf, rm_conf)
    ret = pp.clean_content(test_data, [])
    # file is NOT changed
    assert test_data == ret


@mark.parametrize("obfuscate", [True, False])
def test_redact_exclude_none(obfuscate):
    conf = InsightsConfig(obfuscate=obfuscate)

    pp = Cleaner(conf, None)
    ret = pp.clean_content(test_data, [])
    # file is NOT changed
    assert test_data == ret



from insights.parsers import ls_var_lib_rpm
from insights.parsers.ls_var_lib_rpm import LsVarLibRpm
from insights.tests import context_wrap
import doctest

LS_VAR_LIB_RPM = """
total 172400
drwxr-xr-x.  2 0 0      4096 Oct 19  2022 .
drwxr-xr-x. 31 0 0      4096 Nov 26  2021 ..
-rw-r--r--.  1 0 0   4034560 Apr 17 16:06 Basenames
-rw-r--r--.  1 0 0      8192 Apr 17 16:06 Conflictname
-rw-r--r--.  1 0 0    270336 Apr 24 10:31 __db.001
-rw-r--r--.  1 0 0     81920 Apr 24 10:31 __db.002
-rw-r--r--.  1 0 0   1318912 Apr 24 10:31 __db.003
-rw-r--r--.  1 0 0         0 Nov  9  2021 .dbenv.lock
-rw-r--r--.  1 0 0   2707456 Apr 17 16:06 Dirnames
-rw-r--r--.  1 0 0      8192 Nov  9  2021 Enhancename
-rw-r--r--.  1 0 0      8192 Nov 26  2021 Filetriggername
-rw-r--r--.  1 0 0     12288 Apr 17 16:06 Group
-rw-r--r--.  1 0 0     20480 Apr 17 16:06 Installtid
-rw-r--r--.  1 0 0     45056 Apr 17 16:06 Name
-rw-r--r--.  1 0 0     16384 Apr 17 10:43 Obsoletename
-rw-r--r--.  1 0 0 165253120 Apr 17 16:06 Packages
-rw-r--r--.  1 0 0   2461696 Apr 17 16:06 Providename
-rw-r--r--.  1 0 0      8192 Apr 17 10:42 Recommendname
-rw-r--r--.  1 0 0    245760 Apr 17 16:06 Requirename
-rw-r--r--.  1 0 0         0 Nov  9  2021 .rpm.lock
-rw-r--r--.  1 0 0     77824 Apr 17 16:06 Sha1header
-rw-r--r--.  1 0 0     40960 Apr 17 16:06 Sigmd5
-rw-r--r--.  1 0 0      8192 Apr 17 10:42 Suggestname
-rw-r--r--.  1 0 0      8192 Apr 17 10:42 Supplementname
-rw-r--r--.  1 0 0      8192 Apr 17 10:42 Transfiletriggername
-rw-r--r--.  1 0 0      8192 Apr 17 10:44 Triggername
"""

LS_VAR_LIB_RPM_DOC_EXAMPLE = """
total 172400
drwxr-xr-x.  2 0 0      4096 Oct 19  2022 .
drwxr-xr-x. 31 0 0      4096 Nov 26  2021 ..
-rw-r--r--.  1 0 0   4034560 Apr 17 16:06 Basenames
-rw-r--r--.  1 0 0      8192 Apr 17 16:06 Conflictname
-rw-r--r--.  1 0 0     16384 Apr 17 10:43 Obsoletename
-rw-r--r--.  1 0 0 165253120 Apr 17 16:06 Packages
-rw-r--r--.  1 0 0   2461696 Apr 17 16:06 Providename
-rw-r--r--.  1 0 0      8192 Apr 17 10:42 Recommendname
"""


def test_ls_var_lib_rpm():
    var_lib_rpm = LsVarLibRpm(context_wrap(LS_VAR_LIB_RPM, path="insights_commands/ls_-lan_.var.lib.rpm"))
    assert len(var_lib_rpm.files_of('/var/lib/rpm')) == 24
    assert 'Recommendname' in var_lib_rpm.files_of('/var/lib/rpm')
    assert var_lib_rpm.dir_contains('/var/lib/rpm', 'Packages') is True
    assert var_lib_rpm.dir_entry('/var/lib/rpm', 'Triggername') == {'type': '-', 'perms': 'rw-r--r--.', 'links': 1, 'owner': '0', 'group': '0', 'size': 8192, 'date': 'Apr 17 10:44', 'name': 'Triggername', 'raw_entry': '-rw-r--r--.  1 0 0      8192 Apr 17 10:44 Triggername', 'dir': '/var/lib/rpm'}


def test_ls_var_lib_rpm_doc_examples():
    env = {
        'var_lib_rpm': LsVarLibRpm(context_wrap(LS_VAR_LIB_RPM_DOC_EXAMPLE, path="insights_commands/ls_-lan_.var.lib.rpm"))
           }
    failed, total = doctest.testmod(ls_var_lib_rpm, globs=env)
    assert failed == 0



"""
pluginconf.d configuration file - Files
=======================================

Shared mappers for parsing and extracting data from
``/etc/yum/pluginconf.d/*.conf`` files. Parsers contained
in this module are:

PluginConfDIni - files ``/etc/yum/pluginconf.d/*.conf``
-------------------------------------------------------
"""
from insights.core import IniConfigFile
from insights.core.plugins import parser
from insights.specs import Specs


@parser(Specs.pluginconf_d)
class PluginConfDIni(IniConfigFile):
    """
    Read yum plugin config files, in INI format, using the standard INI file
    parser class.

    Sample configuration::

        [main]
        enabled = 0
        gpgcheck = 1
        timeout = 120

        # You can specify options per channel, e.g.:
        #
        #[rhel-i386-server-5]
        #enabled = 1
        #
        #[some-unsigned-custom-channel]
        #gpgcheck = 0

        [test]
        test_multiline_config = http://example.com/repos/test/
                                http://mirror_example.com/repos/test/

    Examples:
        >>> type(conf)
        <class 'insights.parsers.pluginconf_d.PluginConfDIni'>
        >>> conf.sections()
        ['main', 'test']
        >>> conf.has_option('main', 'gpgcheck')
        True
        >>> conf.get("main", "enabled")
        '0'
        >>> conf.getint("main", "timeout")
        120
        >>> conf.getboolean("main", "enabled")
        False
        >>> conf.get("test", "test_multiline_config")
        'http://example.com/repos/test/ http://mirror_example.com/repos/test/'
    """
    pass



"""
Nftables Configurations
=======================

This module includes the following parsers:

NftListRuleSet - command ``nft -j list ruleset``
------------------------------------------------
"""

from insights.core import JSONParser
from insights.core.dr import SkipComponent
from insights.core.plugins import parser
from insights.specs import Specs
from insights.parsr import query
from insights.parsr.query import make_child_query as q


@parser(Specs.nft_list_ruleset)
class NftListRuleSet(JSONParser):
    """
    It parses the output of "nft -j list ruleset".

    Sample output::

        {
            "nftables": [
                {
                    "metainfo": {"version": "0.9.3", "release_name": "Topsy", "json_schema_version": 1}
                },
                {
                    "table": {"family": "ip", "name": "table1"}
                },
                {
                    "map": {
                        "family": "ip", "name": "example_map", "table": "table1",
                        "type": "ipv4_addr", "map": "verdict",
                        "elem": [
                            ["192.0.2.1", {"accept": null}],
                            ["192.0.2.2", {"drop": null}],]
                    }
                },
                {
                    "chain": {
                        "family": "ip", "table": "table1", "name": "chain1",
                        "type": "filter", "hook": "input", "prio": 0,
                        "policy": "accept"
                    }
                },
                {
                    "rule": {
                        "family": "ip", "table": "table1", "chain": "chain1",
                        "expr": [
                            {"vmap": {"key": {"payload": {"protocol": "ip", "field": "saddr"}},
                            "data": "@example_map"}}]
                    }
                },
                {
                    "rule": {
                        "family": "ip", "table": "table1", "chain": "chain1",
                        "expr": [
                            {"match": {
                                "op": "==",
                                "left": {"payload": {"protocol": "tcp", "field": "dport"}},
                                "right": 22}}
                        ]
                    }
                }
            ]
        }

    The json output is dict with a single "nftables" key, the value is a list of all data in nftables.
    To make it easier to use, here it makes use of ``insights.parsr.query.Entry`` and transfer the
    data to an Entry object and set it back to the data attribute.

    Attributes:
        data (object): An ``insights.parsr.query.Entry`` object containing all of the data as its
            children.

    Raises:
        SkipComponent: No tables available.

    Examples:
        >>> type(nft_obj)
        <class 'insights.parsers.nftables.NftListRuleSet'>
        >>> [str(item) for item in nft_obj.tables('ip')]   # change unicode to string to be compatible with python2.6/2.7
        ['table1']
        >>> chains = nft_obj.chains('ip', 'table1')
        >>> len(chains)
        1
        >>> str(chains[0]['name'].value)  # change unicode to string to be compatible with python2.6/2.7
        'chain1'
        >>> str(chains[0]['type'].value)  # change unicode to string to be compatible with python2.6/2.7
        'filter'
        >>> rules = nft_obj.rules('ip', 'table1', 'chain1')
        >>> len(rules)
        2
        >>> rules[1]['expr']['match']['right'].value
        22
    """

    def parse_content(self, content):
        super(NftListRuleSet, self).parse_content(content)
        self.data = query.from_dict(self.data)
        if not self.data['nftables']['table']:
            raise SkipComponent

    def tables(self, address_family):
        """list: Return the table names in some address family."""
        return [item['name'].value for item in self.data['nftables']['table'].where('family', address_family)]

    def chains(self, address_family, table_name):
        """list: Returns the chains in some table."""
        return self.data['nftables']['chain'].where(q('family', address_family) & q('table', table_name))

    def rules(self, address_family, table_name, chain_name):
        """list: Returns the rules of some chain in some table."""
        return self.data['nftables']['rule'].where(q('family', address_family) & q('table', table_name) & q('chain', chain_name))



import doctest

from insights.parsers import leapp
from insights.parsers.leapp import LeappReport, LeappMigrationResults
from insights.tests import context_wrap
from insights.tests.datasources.test_leapp import LEAPP_REPORT_RESULT
from insights.tests.datasources.test_leapp import MIGRATION_RESULTS_RET_1, MIGRATION_RESULTS_RET_2


def test_leapp_report():
    ret = LeappReport(context_wrap(LEAPP_REPORT_RESULT, path='insights_commands/leapp_report'))
    assert len(ret) == 4
    assert ret[0]['title'] == "Use of NFS detected. Upgrade can't proceed"
    assert 'summary' in ret[0]
    assert ret[1]['title'] == "Possible problems with remote login using root account"
    assert 'summary' in ret[1]
    assert ret[0]['remediations'][0]['context'] == "Disable NFS temporarily for the upgrade if possible."
    assert 'If you depend on remote' in ret[1]['remediations'][0]['context']
    assert 'remediations' not in ret[2]
    assert 'external' in ret[2]
    assert 'remediations' not in ret[3]
    assert 'related_resources' in ret[3]


def test_leapp_migration_results():
    ret = LeappMigrationResults(context_wrap(MIGRATION_RESULTS_RET_1, path='insights_commands/leapp_migration_results'))
    assert len(ret) == 2
    assert ret[0]['activity_ended'] == "2023-08-22T08:56:26.971009Z"
    assert ret[0]['run_id'] == "1edff870-626d-41ba-854c-8f9dc8f20dc3"
    assert ret[1]['target_os'] == "Red Hat Enterprise Linux 9.0"
    assert ret[1]['env']['LEAPP_CURRENT_PHASE'] == "InterimPreparation"


def test_c2r_migration_results():
    ret = LeappMigrationResults(context_wrap(MIGRATION_RESULTS_RET_2, path='insights_commands/leapp_migration_results'))
    assert len(ret) == 1
    assert ret[0]['activity_ended'] == "2024-03-01T07:44:15.814899Z"
    assert ret[0]['run_id'] == "null"
    assert ret[0]['target_os'] == "null"
    assert len(ret[0]['env']) == 0


def test_doc_examples():
    env = {
        'leapp_report': LeappReport(context_wrap(LEAPP_REPORT_RESULT)),
        'leapp_migration_results': LeappMigrationResults(context_wrap(MIGRATION_RESULTS_RET_1)),
    }
    failed, total = doctest.testmod(leapp, globs=env)
    assert failed == 0



"""
NetworkNamespace = ``/bin/ls /var/run/netns``
=============================================

This specs provides list of network namespace created on the host machine.

Typical output of this command is as below::

    temp_netns  temp_netns_2  temp_netns_3


The ``/bin/ls /var/run/netns`` is prefered over ``/bin/ip netns list`` because it works on
all RHEL versions, no matter ip package is installed or not.

Examples:
    >>> type(netns_obj)
    <class 'insights.parsers.net_namespace.NetworkNamespace'>
    >>> netns_obj.netns_list
    ['temp_netns', 'temp_netns_2', 'temp_netns_3']
    >>> len(netns_obj.netns_list)
    3
"""
from insights.core import Parser
from insights.core.exceptions import SkipComponent
from insights.core.plugins import parser
from insights.parsers import get_active_lines
from insights.specs import Specs


@parser(Specs.namespace)
class NetworkNamespace(Parser):
    def parse_content(self, content):
        if not content:
            raise SkipComponent('Nothing to parse.')

        self._netns_list = []
        for line in get_active_lines(content):
            self._netns_list.extend(line.split())

    @property
    def netns_list(self):
        """
        This method returns list of network namespace created
        in process memory.

        Returns:

            `list` of network namepaces if exists.
        """
        return self._netns_list



"""
Custom datasource to get the duplicate machine info.
"""
import json

from insights.core.plugins import datasource
from insights.core.spec_factory import DatasourceProvider
from insights.core.filters import get_filters
from insights.core.context import HostContext
from insights.core.exceptions import SkipComponent
from insights.client.client import get_connection
from insights.specs import Specs


@datasource(HostContext, Specs.machine_id)
def dup_machine_id_info(broker):
    """
    This datasource provides the duplicate machine info.

    Sample Output::

        dc194312-8cdd-4e75-8cf1-2094bf666f45 hostname1,hostname2

    Returns:
        str: a string containing the machine id and the hostnames with the same machine id

    Raises:
        SkipComponent: When the filters does not exist or the machine id is not
           in the filters or the machine id is not duplicate or any exception
           occurs.
    """
    filters = sorted((get_filters(Specs.duplicate_machine_id)))
    if not filters:
        raise SkipComponent
    machine_id_obj = broker[Specs.machine_id]
    if len(machine_id_obj.content) == 1:
        machine_id = str(machine_id_obj.content[0].strip())
        if machine_id in filters:
            config = broker.get('client_config')
            try:
                conn = get_connection(config)
                if config.legacy_upload:
                    url = conn.base_url + '/platform/inventory/v1/hosts?insights_id=' + machine_id
                else:
                    url = conn.inventory_url + '/hosts?insights_id=' + machine_id
                res = conn.get(url)
                res_json = json.loads(res.content)
            except Exception:
                raise SkipComponent
            if res_json['total'] > 1:
                duplicate_hostnames = [item.get('fqdn') for item in res_json['results'] if item.get('fqdn')]
                content = '%s %s' % (machine_id, ','.join(duplicate_hostnames))
                return DatasourceProvider(content=[content], relative_path='insights_commands/duplicate_machine_id_info')
    raise SkipComponent



import os
import tempfile
import yaml

from mock.mock import Mock

from insights.collect import (
        load_manifest,
        default_manifest,
        generate_archive_name,
        _parse_broker_exceptions)
from insights.core.dr import Broker
from insights.core.exceptions import ContentException


def test_parse_broker_exceptions_no_errors():
    exc_to_report = set([OSError])
    broker = Broker()
    errors = _parse_broker_exceptions(broker, exc_to_report)
    assert not errors


def test_parse_broker_exceptions_oserror():
    exc_to_report = set([OSError])
    broker = Broker()
    component_one = Mock()
    broker.add_exception(component_one, OSError("OS Error occurred"))
    component_two = Mock()
    broker.add_exception(component_two, OSError("OS Error occurred again"))
    errors = _parse_broker_exceptions(broker, exc_to_report)
    assert OSError in errors.keys()
    assert len(errors[OSError]) == 2
    assert all(type(ex) is OSError for ex, comp in errors[OSError])


def test_parse_broker_exceptions_oserror_contentexception():
    exc_to_report = set([OSError])
    broker = Broker()
    component_one = Mock()
    broker.add_exception(component_one, OSError("OS Error occurred"))
    component_two = Mock()
    broker.add_exception(component_two, ContentException("Bad content"))
    errors = _parse_broker_exceptions(broker, exc_to_report)
    assert OSError in errors.keys()
    assert ContentException not in errors.keys()
    assert len(errors[OSError]) == 1
    assert all(type(ex) is OSError for ex, comp in errors[OSError])


def test_load_manifest():
    # dict
    data = yaml.safe_load(default_manifest)
    ret = load_manifest(data)
    assert ret == data
    # string
    ret = load_manifest(default_manifest)
    assert ret == data
    # file
    tmpfile = tempfile.NamedTemporaryFile(prefix='tmp_', suffix='_manifest_rhin', delete=False)
    tmpfile.close()
    with open(tmpfile.name, 'w') as fd:
        fd.write(default_manifest)
    ret = load_manifest(tmpfile.name)
    assert ret == data
    os.remove(tmpfile.name)


def test_generate_archive_name():
    archive_name = generate_archive_name()
    assert archive_name.startswith("insights-")



"""
LsPci - Commands ``lspci``
==========================

This combiner combines the following Parsers to a list.
- LsPci - the output of command ``lspci -k``
- LsPciVmmkn - the output of command ``lspci -vmmkn``
"""
from insights import combiner
from insights.parsers import keyword_search
from insights.parsers.lspci import LsPci, LsPciVmmkn


@combiner([LsPci, LsPciVmmkn])
class LsPci(list):
    """
    Combines the Parser LsPci of ``/sbin/lspci -k`` command and Parser
    LsPciVmmkn of ``/sbin/lspci -vmmkn`` command.

    .. note::
        In case the ``lspci -k`` sometimes outputs the `Slot` in the full
        format of ``domain:bus:device.function``, and the ``lspci -k`` is more
        common than ``lspci -vmmkn``, so this combiner will take the `Slot` of
        the `lspci -k` as the key.

    Typical output of the ``lspci -k`` command is::

        00:00.0 Host bridge: Intel Corporation Haswell-ULT DRAM Controller (rev 09)
                Subsystem: Lenovo ThinkPad X240
                Kernel driver in use: hsw_uncore
        00:02.0 VGA compatible controller: Intel Corporation Haswell-ULT Integrated Graphics Controller (rev 09)
                Subsystem: Lenovo ThinkPad X240
                Kernel driver in use: i915
                Kernel modules: i915
        00:03.0 Audio device: Intel Corporation Haswell-ULT HD Audio Controller (rev 09)
                Subsystem: Lenovo ThinkPad X240
                Kernel driver in use: snd_hda_intel
                Kernel modules: snd_hda_intel
        00:16.0 Communication controller: Intel Corporation 8 Series HECI #0 (rev 04)
                Subsystem: Lenovo ThinkPad X240
                Kernel driver in use: mei_me
                Kernel modules: mei_me
        00:19.0 Ethernet controller: Intel Corporation Ethernet Connection I218-LM (rev 04)
                Subsystem: Lenovo ThinkPad X240
                Kernel driver in use: e1000e
                Kernel modules: e1000e
        00:1b.0 Audio device: Intel Corporation 8 Series HD Audio Controller (rev 04)
                Subsystem: Lenovo ThinkPad X240
                Kernel driver in use: snd_hda_intel
                Kernel modules: snd_hda_intel

    Typical output of the ``lspci -vmmkn`` command is::

        Slot:   00:00.0
        Class:  0600
        Vendor: 8086
        Device: 0a04
        SVendor:        17aa
        SDevice:        2214
        Rev:    09
        Driver: hsw_uncore

        Slot:   00:02.0
        Class:  0300
        Vendor: 8086
        Device: 0a16
        SVendor:        17aa
        SDevice:        2214
        Rev:    09
        Driver: i915
        Module: i915

        Slot:   00:03.0
        Class:  0403
        Vendor: 8086
        Device: 0a0c
        SVendor:        17aa
        SDevice:        2214
        Rev:    09
        Driver: snd_hda_intel
        Module: snd_hda_intel

        Slot:   00:16.0
        Class:  0780
        Vendor: 8086
        Device: 9c3a
        SVendor:        17aa
        SDevice:        2214
        Rev:    04
        Driver: mei_me
        Module: mei_me

        Slot:   00:19.0
        Class:  0200
        Vendor: 8086
        Device: 155a
        SVendor:        17aa
        SDevice:        2214
        Rev:    04
        Driver: e1000e
        Module: e1000e

        Slot:   00:1b.0
        Class:  0403
        Vendor: 8086
        Device: 9c20
        SVendor:        17aa
        SDevice:        2214
        Rev:    04
        Driver: snd_hda_intel
        Module: snd_hda_intel

    Examples:
        >>> type(lspci)
        <class 'insights.combiners.lspci.LsPci'>
        >>> sorted(lspci.pci_dev_list)
        ['00:00.0', '00:02.0', '00:03.0', '00:16.0', '00:19.0', '00:1b.0']
        >>> lspci.search(Dev_Details__contains='I218')[0]['Slot']
        '00:19.0'
    """
    def __init__(self, lspci_k, lspci_vmmkn):
        if lspci_vmmkn:
            for dev in lspci_vmmkn:
                # use the local copy to prevent from writing back to the parser
                dev_t = dev.copy()
                if lspci_k and (dev['Slot'] in lspci_k.data or '0000:' + dev['Slot'] in lspci_k.data):
                    # use the local copy to prevent from writing back to the parser
                    dev_k = [v for v in lspci_k.data.values() if v['Slot'].endswith(dev['Slot'])][0].copy()
                    # Since the 'lspci -k' is a more common command than the
                    #  'lspci -vmmkn', the following line should be commented
                    #  out to use the 'Slot' in 'lspci -k' as the 'Slot' in
                    #  this combiner:
                    # dev_k.pop('Slot') if 'Slot' in dev_k else None
                    dev_k.pop('Kernel driver in use') if 'Kernel driver in use' in dev_k else None
                    dev_k.pop('Kernel modules') if 'Kernel modules' in dev_k else None
                    dev_t.update(dev_k)

                self.append(dev_t)
            self._pci_dev_list = (lspci_k if lspci_k else lspci_vmmkn).pci_dev_list
        else:
            for dev in lspci_k.data.values():
                # use the local copy to prevent from writing back to the parser
                dev = dev.copy()
                dev.update(Driver=dev.pop('Kernel driver in use')) if 'Kernel driver in use' in dev else None
                dev.update(Module=[i.strip() for i in dev.pop('Kernel modules').split(',')]) if 'Kernel modules' in dev else None
                self.append(dev)
            self._pci_dev_list = lspci_k.pci_dev_list

    @property
    def pci_dev_list(self):
        """
        The list of PCI devices.
        """
        return self._pci_dev_list

    def search(self, **kwargs):
        """
        Get the details of PCI devices by searching the table with kwargs.

        This uses the :py:func:`insights.parsers.keyword_search` function for
        searching; see its documentation for usage details. If no search
        parameters are given, no rows are returned.

        It simplify the value of the column according to actual usage.

        Returns:
            list: A list of dictionaries of PCI devices that match the given
            search criteria.

        Examples:
            >>> len(lspci.search(Subsystem__startswith='Lenovo'))
            6
            >>> len(lspci.search(Subsystem__startswith='Lenovo', Dev_Details__startswith='Audio device'))
            2
            >>> lspci.search(Driver='snd_hda_intel', Dev_Details__contains='8') == [
            ... {'Slot': '00:1b.0', 'Class': '0403', 'Vendor': '8086',
            ...  'Device': '9c20', 'SVendor': '17aa', 'SDevice': '2214',
            ...  'Rev': '04', 'Driver': 'snd_hda_intel',
            ...  'Module': ['snd_hda_intel'], 'Subsystem': 'Lenovo ThinkPad X240',
            ...  'Dev_Details': 'Audio device: Intel Corporation 8 Series HD Audio Controller (rev 04)'}]
            True
        """
        return keyword_search(self, **kwargs)



import doctest
import pytest

from insights.core.exceptions import SkipComponent
from insights.parsers import readlink_openshift_certs
from insights.tests import context_wrap

CLIENT_REAL_FILE_PATH = '''
/etc/origin/node/certificates/kubelet-client-2019-10-18-23-17-35.pem
'''.strip()

SERVER_REAL_FILE_PATH = '''
/etc/origin/node/certificates/kubelet-server-2018-10-18-23-29-14.pem
'''.strip()

BAD_FILE_PATH = ""


def test_doc_examples():
    env = {
        'client': readlink_openshift_certs.ReadLinkEKubeletClientCurrent(
            context_wrap(CLIENT_REAL_FILE_PATH)),
        'server': readlink_openshift_certs.ReadLinkEKubeletServerCurrent(
            context_wrap(SERVER_REAL_FILE_PATH)),
    }
    failed, total = doctest.testmod(readlink_openshift_certs, globs=env)
    assert failed == 0


def test_readlink_openshift_certs():
    client = readlink_openshift_certs.ReadLinkEKubeletClientCurrent(context_wrap(CLIENT_REAL_FILE_PATH))
    assert len(client.path) > 0
    assert client.path == CLIENT_REAL_FILE_PATH

    server = readlink_openshift_certs.ReadLinkEKubeletServerCurrent(context_wrap(SERVER_REAL_FILE_PATH))
    assert len(server.path) > 0
    assert server.path == SERVER_REAL_FILE_PATH


def test_fail():
    with pytest.raises(SkipComponent) as e:
        readlink_openshift_certs.ReadLinkEKubeletClientCurrent(context_wrap(BAD_FILE_PATH))
    assert "No Data from command: /usr/bin/readlink -e /etc/origin/node/certificates/kubelet-client-current.pem" in str(e)

    with pytest.raises(SkipComponent) as e:
        readlink_openshift_certs.ReadLinkEKubeletServerCurrent(context_wrap(BAD_FILE_PATH))
    assert "No Data from command: /usr/bin/readlink -e /etc/origin/node/certificates/kubelet-server-current.pem" in str(e)



"""
IpNetnsExecNamespaceLsofI - command ``/sbin/ip netns exec [network-namespace] lsof -i"``
========================================================================================

This module provides class ``IpNetnsExecNamespaceLsofI`` for parsing the output of command
``/sbin/ip netns exec [network-namespace] lsof -i``.
Filters have been added so that sensitive information can be filtered out.
This results in the modification of the original structure of data.
"""
from collections import namedtuple

from insights.core import CommandParser
from insights.core.exceptions import SkipComponent
from insights.core.filters import add_filter
from insights.core.plugins import parser
from insights.parsers import keyword_search, parse_fixed_table
from insights.specs import Specs

add_filter(Specs.ip_netns_exec_namespace_lsof, "COMMAND")


@parser(Specs.ip_netns_exec_namespace_lsof)
class IpNetnsExecNamespaceLsofI(CommandParser):
    """
       This class provides processing for the output of command
       ``/sbin/ip netns exec [network-namespace] lsof -i``.

       Sample command output::

           COMMAND   PID   USER    FD  TYPE  DEVICE     SIZE/OFF  NODE NAME
           neutron-n 975   root    5u  IPv4  6482691    0t0        TCP *:http (LISTEN)

       Examples:
           >>> len(ns_lsof.search(command="neutron-n"))
           1
           >>> ns_lsof.data[0]["command"] == "neutron-n"
           True

       Attributes:
           fields (list): List of ``KeyValue`` namedtupules for each line
                          in the command.

           data (list): List of key value pair derived from the command.

       Raises:
           SkipComponent: When the file is empty or data is useless.
    """
    keyvalue = namedtuple("KeyValue",
                          ["command", "pid", "user", "fd", "type", "device", "size_off", "node", "name"])

    def parse_content(self, content):
        self.fields = []
        self.data = []
        if not content:
            raise SkipComponent("Empty file")

        self.data = parse_fixed_table(content,
                                      heading_ignore=["COMMAND", "PID", "USER", "FD", "TYPE", "DEVICE", "SIZE/OFF", "NODE", "NAME"],
                                      header_substitute=[("COMMAND", "command"), ("PID", "pid"), ("USER", "user"), ("FD", "fd"),
                                                         ("TYPE", "type"), ("DEVICE", "device"), ("SIZE/OFF", "size_off"),
                                                         ("NODE", "node"), ("NAME", "name")])
        if not self.data:
            raise SkipComponent("Useless data")

        for item in self.data:
            self.fields.append(self.keyvalue(item["command"], item["pid"], item["user"], item["fd"], item["type"], item["device"],
                               item["size_off"], item["node"], item["name"]))

    def __iter__(self):
        return iter(self.fields)

    def search(self, **kw):
        """Search item based on key value pair.

        Example:

            >>> len(ns_lsof.search(command="neutron-n")) == 1
            True
            >>> len(ns_lsof.search(user="nobody")) == 0
            True
        """
        return keyword_search(self.data, **kw)



"""
DseLDIF - file ``/etc/dirsrv/*/dse.ldif``
=========================================
"""

from collections import defaultdict

from insights import Parser, parser
from insights.core.exceptions import SkipComponent
from insights.core.filters import add_filter
from insights.specs import Specs

add_filter(Specs.dse_ldif, 'dn:')


@parser(Specs.dse_ldif)
class DseLDIF(Parser, list):
    """
    Parse the attributes out of the filtered lines of the
    ``/etc/dirsrv/*/dse.ldif`` file into a list of records. Each record will
    be stored as a defaultdict(list), take attribute name as the dict key, and
    wrap the attribute's values into a list as the dict value.

    The ``dse.ldif`` files are in the LDIF format (see ``man 5 ldif``). Due
    to the file content are filtered, there are some differences between the
    formal LDIF format and this parser.

    LDIF contains multi-row records where each record is identified by a
    ``dn:`` line ("dn" as in "distinguished name") and the record's other lines
    are attributes. Attributes may also have base64-encoded values, multiline
    values, and file-stored values.

    LDIF supports multiline values via continuing lines, one can break and
    continue a line or fold a line by indenting the continued portion of the
    line by one space.

    Since the content to be handled here is filtered content, the LDIF's
    continuing lines will be broken and no more trustable. When parsing, this
    parser will drop the lines startswith one space. This is acceptable for the
    current usage of the filter content. We can see to workaround this if
    continuing line requirement comes in the future.

    In the parsing process, line for ``dn:`` attribute will be recognized as
    the start of a record, the following lines will be included in this same
    record.

    Therefor, "dn:" is a default filter for the content collection. And a
    suggested way for filter adding is using the attribute key with ":" to
    minimize the collected data, eg. ``add_filter(Specs.dse_ldif, 'nsSSL3:')``.

    Sample input data::

        dn: cn=config
        nsslapd-return-default-opattr: namingContexts
        nsslapd-return-default-opattr: supportedControl
        nsslapd-securePort: 636
        nsslapd-security: on

        dn: cn=encryption,cn=config
        sslVersionMin: SSLv3
        sslVersionMax: TLS1.1
        nsSSL3: on

    Examples:
        >>> type(dse_ldif)
        <class 'insights.parsers.dse_ldif.DseLDIF'>
        >>> len(dse_ldif)
        2
        >>> dse_ldif[0]["nsslapd-return-default-opattr"]
        ['namingContexts', 'supportedControl']
        >>> dse_ldif[0]["nsslapd-security"]
        ['on']
        >>> "sslVersionMin" in dse_ldif[1]
        True
        >>> dse_ldif[1]["nsSSL3"][0]
        'on'

    """

    def parse_content(self, content):
        if not content:
            raise SkipComponent('Empty file content')

        records = []
        attr_kval = defaultdict(list)
        for line in content:

            # empty lines or lines beginning with # are ignored
            if not line or line.startswith("#"):
                continue
            #  the continuing lines are ignored
            if line.startswith(" "):
                continue
            # lines not in attributes format are ignored
            if ":" not in line:
                continue

            attr_name, attr_value = [_.strip() for _ in line.split(':', 1)]
            if attr_name == 'dn':
                # line starts with 'dn:', taken as the start of a new record
                attr_kval = defaultdict(list)
                records.append(attr_kval)
                attr_kval[attr_name].append(attr_value)
            else:
                if attr_value.startswith(":"):
                    # base64-encrypted values not supported
                    continue
                if attr_value.startswith("<"):
                    # file-backed values not supported
                    continue
                attr_kval[attr_name].append(attr_value)

        for record in records:
            self.append(dict(record))

        if not self:
            raise SkipComponent("No valid content")



"""
saphostexec - Commands
======================

Shared parsers for parsing output of the ``saphostexec [option]`` commands.

SAPHostExecStatus - command ``saphostexec -status``
---------------------------------------------------

SAPHostExecVersion - command ``saphostexec -version``
-----------------------------------------------------
"""
from collections import namedtuple

from insights.core import CommandParser
from insights.core.exceptions import ParseException, SkipComponent
from insights.core.plugins import parser
from insights.specs import Specs


@parser(Specs.saphostexec_status)
class SAPHostExecStatus(CommandParser, dict):
    """
    Class for parsing the output of ``saphostexec -status`` command.

    Typical output of the command is::

        saphostexec running (pid = 9159)
        sapstartsrv running (pid = 9163)
        saposcol running (pid = 9323)

    Examples:
        >>> type(sha_status)
        <class 'insights.parsers.saphostexec.SAPHostExecStatus'>
        >>> sha_status.is_running
        True
        >>> sha_status.services['saphostexec'].status
        'running'
        >>> sha_status['saphostexec'].status
        'running'
        >>> sha_status['saphostexec'].pid
        '9159'
    """

    SAPHostAgentService = namedtuple("SAPHostAgentService", field_names=["status", "pid"])
    """namedtuple: Type for storing the lines of ``saphostexec -status``"""

    def parse_content(self, content):
        data = {}
        for line in content:
            if not line.strip():
                continue
            line_sp = line.strip().split(None, 1)
            if len(line_sp) == 2:
                value_sp = line_sp[1].replace('(', '').replace(')', '').split()
                svc, sta, pid = line_sp[0], value_sp[0], value_sp[-1]
                data[svc] = self.SAPHostAgentService(sta, pid)
            else:
                raise ParseException("Incorrect line: '{0}'".format(line))
        if data:
            self.update(data)
        else:
            raise SkipComponent

    @property
    def is_running(self):
        """
        Returns if the SAPHostAgent is running or not.
        """
        return all(p.status == 'running' for p in self.values())

    @property
    def data(self):
        """
        .. warning::

            Deprecated, the parser works as a dict please use the built-in
            accesses of `dict`

        Returns the parsed data.
        """
        return self

    @property
    def services(self):
        """
        Returns the parsed lines.
        """
        return self


@parser(Specs.saphostexec_version)
class SAPHostExecVersion(CommandParser, dict):
    """
    Class for parsing the output of ``saphostexec -version`` command.

    Typical output of the command is::

        *************************** Component ********************
        /usr/sap/hostctrl/exe/saphostexec: 721, patch 1011, changelist 1814854, linuxx86_64, opt (Jan 13 2018, 04:43:56)
        /usr/sap/hostctrl/exe/sapstartsrv: 721, patch 1011, changelist 1814854, linuxx86_64, opt (Jan 13 2018, 04:43:56)
        /usr/sap/hostctrl/exe/saphostctrl: 721, patch 1011, changelist 1814854, linuxx86_64, opt (Jan 13 2018, 04:43:56)
        /usr/sap/hostctrl/exe/xml71d.so: 721, patch 1011, changelist 1814854, linuxx86_64, opt (Jan 13 2018, 01:12:10)
        **********************************************************
        --------------------
        SAPHOSTAGENT information
        --------------------
        kernel release                721
        kernel make variant           721_REL
        compiled on                   Linux GNU SLES-9 x86_64 cc4.1.2  for linuxx86_64
        compiled for                  64 BIT
        compilation mode              Non-Unicode
        compile time                  Jan 13 2018 04:40:52
        patch number                  33
        latest change number          1814854
        ---------------------
        supported environment
        ---------------------
        operating system
        Linux 2.6
        Linux 3
        Linux

    Examples:
        >>> type(sha_version)
        <class 'insights.parsers.saphostexec.SAPHostExecVersion'>
        >>> sha_version.components['saphostexec'].version
        '721'
        >>> sha_version['saphostexec'].version
        '721'
        >>> sha_version['saphostexec'].patch
        '1011'
    """

    SAPHostAgentComponent = namedtuple("SAPHostAgentComponent",
            field_names=["version", "patch", "changelist"])
    """namedtuple: Type for storing the lines of ``saphostexec -version``"""

    def parse_content(self, content):
        data = {}
        for line in content:
            # Only process component lines for now
            if not line.startswith('/usr/sap/hostctrl/exe/'):
                continue
            key, val = line.split(':', 1)
            key = key.split('/')[-1]
            ver, pch, chl, _ = [s.split()[-1].strip() for s in val.split(', ', 3)]
            data[key] = self.SAPHostAgentComponent(ver, pch, chl)
        if data:
            self.update(data)
        else:
            raise SkipComponent

    @property
    def data(self):
        """
        .. warning::

            Deprecated, the parser works as a dict please use the built-in
            accesses of `dict`

        Returns the parsed data.
        """
        return self

    @property
    def components(self):
        """
        Return the dict of :py:class:`SAPHostAgentComponent` instances.
        """
        return self



"""
IPCS Shared Memory Segments
===========================

Combiner for parsing shared memory segments gotten from command ``ipcs``. It
uses the results of the ``IpcsM`` and ``IpcsMP`` parsers to get the size of the
shared memory of special ``PID``.

"""
from insights.core import LegacyItemAccess
from insights.core.exceptions import ParseException
from insights.core.plugins import combiner
from insights.parsers.ipcs import IpcsM, IpcsMP


@combiner(IpcsM, IpcsMP)
class IpcsSharedMemory(LegacyItemAccess):
    """
    Class for parsing shared memory segments outputted by commands ``ipcs -m``
    and ``ipcs -m -p``.

    Typical output of command ``ipcs -m`` is::

        ------ Shared Memory Segments --------
        key        shmid      owner      perms      bytes      nattch     status
        0x0052e2c1 0          postgres   600        37879808   26
        0x0052e2c2 1          postgres   600        41222144   24

    Typical output of command ``ipcs -m -p`` is::

        ------ Shared Memory Creator/Last-op --------
        shmid      owner      cpid       lpid
        0          postgres   1833       23566
        1          postgres   1105       9882

    Examples:
        >>> type(ism)
        <class 'insights.combiners.ipcs_shared_memory.IpcsSharedMemory'>
        >>> ism.get_shm_size_of_pid('1105')
        41222144
    """
    def __init__(self, shm, shmp):
        if shm.data.keys() != shmp.data.keys():
            raise ParseException("The output of 'ipcs -m' doesn't match with 'ipcs -m -p'.")
        self.data = {}
        for s, v in shm.data.items():
            self.data[s] = v.copy()
            self.data[s].update(shmp[s])

    def get_shm_size_of_pid(self, pid):
        """
        Return the shared memory size of specified ``pid``.

        Returns:
            (int): size of the shared memory, 0 by default.
        """
        for _, v in self.data.items():
            if v['cpid'] == pid:
                return int(v['bytes'])
        return 0



from __future__ import print_function
from __future__ import absolute_import
from os.path import isfile
import sys
import json
import logging
import logging.handlers
import os
import time
import six
from distutils.version import LooseVersion

from .utilities import (generate_machine_id,
                        write_to_disk,
                        write_registered_file,
                        write_unregistered_file,
                        delete_cache_files,
                        determine_hostname,
                        get_version_info)
from .collection_rules import InsightsUploadConf
from .core_collector import CoreCollector
from .connection import InsightsConnection
from .support import registration_check
from .constants import InsightsConstants as constants

NETWORK = constants.custom_network_log_level
LOG_FORMAT = ("%(asctime)s %(levelname)8s %(name)s:%(lineno)s %(message)s")
logger = logging.getLogger(__name__)


class RotatingFileHandlerWithUMask(logging.handlers.RotatingFileHandler, object):
    """ logging.handlers.RotatingFileHandler subclass with a modified
        file permission mask.
    """
    def __init__(self, umask, *args, **kwargs):
        self._umask = umask
        super(RotatingFileHandlerWithUMask, self).__init__(*args, **kwargs)

    def _open(self):
        """
        Overrides the logging library "_open" method with a custom
        file permission mask.
        """
        default_umask = os.umask(self._umask)
        try:
            return super(RotatingFileHandlerWithUMask, self)._open()
        finally:
            os.umask(default_umask)


class FileHandlerWithUMask(logging.FileHandler, object):
    """ logging.FileHandler subclass with a modified
        file permission mask.
    """
    def __init__(self, umask, *args, **kwargs):
        self._umask = umask
        super(FileHandlerWithUMask, self).__init__(*args, **kwargs)

    def _open(self):
        """
        Overrides the logging library "_open" method with a custom
        file permission mask.
        """
        default_umask = os.umask(self._umask)
        try:
            return super(FileHandlerWithUMask, self)._open()
        finally:
            os.umask(default_umask)


def do_log_rotation():
    handler = get_file_handler()
    return handler.doRollover()


def get_file_handler(config):
    '''
    Sets up the logging file handler.
    Returns:
        RotatingFileHandler - client rpm version is older than 3.2.0.
        FileHandler - client rpm version is 3.2.0 or newer.
    '''
    log_file = config.logging_file
    log_dir = os.path.dirname(log_file)
    if not log_dir:
        log_dir = os.getcwd()
    elif not os.path.exists(log_dir):
        os.makedirs(log_dir, 0o700)
    # ensure the legacy rotating file handler is only used in older client versions
    # or if there is a problem retrieving the rpm version.
    rpm_version = get_version_info()['client_version']
    if not rpm_version or (LooseVersion(rpm_version) < LooseVersion(constants.rpm_version_before_logrotate)):
        file_handler = RotatingFileHandlerWithUMask(0o077, log_file, backupCount=3)
    else:
        file_handler = FileHandlerWithUMask(0o077, log_file)
    file_handler.setFormatter(logging.Formatter(LOG_FORMAT))
    return file_handler


def get_console_handler(config):
    if config.silent:
        target_level = logging.FATAL
    elif config.verbose:
        target_level = logging.DEBUG
    elif config.net_debug:
        target_level = NETWORK
    elif config.quiet:
        target_level = logging.ERROR
    else:
        target_level = logging.INFO

    handler = logging.StreamHandler(sys.stdout)
    handler.setLevel(target_level)

    log_format = LOG_FORMAT if config.verbose else "%(message)s"
    handler.setFormatter(logging.Formatter(log_format))

    return handler


def configure_level(config):
    config_level = 'NETWORK' if config.net_debug else config.loglevel
    config_level = 'DEBUG' if config.verbose else config.loglevel

    init_log_level = logging.getLevelName(config_level)
    if type(init_log_level) in six.string_types:
        print("Invalid log level %s, defaulting to DEBUG" % config_level)
        init_log_level = logging.DEBUG

    logger.setLevel(init_log_level)
    logging.root.setLevel(init_log_level)

    if not config.verbose:
        logging.getLogger('insights.core.dr').setLevel(logging.WARNING)


def set_up_logging(config):
    logging.addLevelName(NETWORK, "NETWORK")
    if len(logging.root.handlers) == 0:
        logging.root.addHandler(get_console_handler(config))
        logging.root.addHandler(get_file_handler(config))
        configure_level(config)
        logger.debug("Logging initialized")


# -LEGACY-
def register(config, pconn):
    """
    Do registration using basic auth
    """
    username = config.username
    password = config.password
    authmethod = config.authmethod
    auto_config = config.auto_config
    if not username and not password and not auto_config and authmethod == 'BASIC':
        logger.debug('Username and password must be defined in configuration file with BASIC authentication method.')
        return False
    return pconn.register()


# -LEGACY-
def _legacy_handle_registration(config, pconn):
    '''
    Handle the registration process
    Returns:
        True - machine is registered
        False - machine is unregistered
        None - could not reach the API
    '''
    logger.debug('Trying registration.')

    # check registration with API
    check = get_registration_status(config, pconn)
    machine_id_present = isfile(constants.machine_id_file)

    if machine_id_present and check['status'] is False:
        logger.info("Machine-id found, insights-client can not be registered."
                    " Please, unregister insights-client first: `insights-client --unregister`")
        return False

    logger.debug('Machine-id: %s', generate_machine_id())

    for m in check['messages']:
        logger.debug(m)

    if check['unreachable']:
        # Run connection test and exit
        return None

    if check['status']:
        # registered in API, resync files
        if config.register:
            logger.info('This host has already been registered.')
        write_registered_file()
        return True

    if config.register:
        # register if specified
        message, hostname, group, display_name = register(config, pconn)
        if not hostname:
            # API could not be reached, run connection test and exit
            logger.error(message)
            return None
        if config.display_name is None and config.group is None:
            logger.info('Successfully registered host %s', hostname)
        elif config.display_name is None:
            logger.info('Successfully registered host %s in group %s',
                        hostname, group)
        else:
            logger.info('Successfully registered host %s as %s in group %s',
                        hostname, display_name, group)
        if message:
            logger.info(message)
        write_registered_file()
        return True
    else:
        # unregistered in API, resync files
        write_unregistered_file(date=check['unreg_date'])
        # print messaging and exit
        if check['unreg_date']:
            # registered and then unregistered
            logger.info('This machine has been unregistered. '
                        'Use --register if you would like to '
                        're-register this machine.')
        else:
            # not yet registered
            logger.info('This machine has not yet been registered. '
                        'Use --register to register this machine.')
        return False


def handle_registration(config, pconn):
    '''
    Does nothing on the platform. Will be deleted eventually.
    '''
    if config.legacy_upload:
        return _legacy_handle_registration(config, pconn)


def get_registration_status(config, pconn):
    '''
        Handle the registration process
        Returns:
            True - machine is registered
            False - machine is unregistered
            None - could not reach the API
    '''
    return registration_check(pconn)


def __cleanup_local_files():
    write_unregistered_file()
    delete_cache_files()
    write_to_disk(constants.machine_id_file, delete=True)
    logger.debug('Unregistered and removed machine-id')


# -LEGACY-
def _legacy_handle_unregistration(config, pconn):
    """
        returns (bool): True success, False failure
    """

    check = get_registration_status(config, pconn)

    for m in check['messages']:
        logger.debug(m)

    if check['unreachable']:
        # Run connection test and exit
        if config.force:
            __cleanup_local_files()
            return True
        return None

    if check['status']:
        unreg = pconn.unregister()
    else:
        unreg = True
        logger.info('This system is already unregistered.')
    if unreg:
        # only set if unreg was successful
        __cleanup_local_files()
        logger.debug('Legacy unregistration')
    return unreg


def handle_unregistration(config, pconn):
    """
    Returns:
        True - machine was successfully unregistered
        False - machine could not be unregistered
        None - could not reach the API
    """
    if config.legacy_upload:
        return _legacy_handle_unregistration(config, pconn)

    unreg = pconn.unregister()
    if unreg or config.force:
        # only set if unreg was successful or --force was set
        __cleanup_local_files()
    return unreg


def get_machine_id():
    return generate_machine_id()


def get_branch_info(config):
    """
    Get branch info for a system
    returns (dict): {'remote_branch': -1, 'remote_leaf': -1}
    """
    # in the case we are running on offline mode
    # or we are analyzing a running container/image
    # or tar file, mountpoint, simply return the default branch info
    if config.offline:
        return constants.default_branch_info
    return config.branch_info


def collect(config):
    """
    All the heavy lifting done here
    """
    pc = InsightsUploadConf(config)
    dc = CoreCollector(config)

    logger.info('Starting to collect Insights data for %s' % determine_hostname(config.display_name))

    dc.run_collection(pc.get_rm_conf(),
                      get_branch_info(config),
                      pc.create_report())

    return dc.done()


def get_connection(config):
    return InsightsConnection(config)


def _legacy_upload(config, pconn, tar_file, content_type, collection_duration=None):
    logger.info('Uploading Insights data.')
    api_response = None
    for tries in range(config.retries):
        logger.debug("Legacy upload attempt %d of %d ...", tries + 1, config.retries)
        try:
            upload = pconn.upload_archive(tar_file, '', collection_duration)
        except Exception as e:
            display_upload_error_and_retry(config, tries, str(e))
            continue

        if upload.status_code in (200, 201):
            api_response = json.loads(upload.text)

            # Write to last upload file
            with open(constants.last_upload_results_file, 'w') as handler:
                if six.PY3:
                    handler.write(upload.text)
                else:
                    handler.write(upload.text.encode('utf-8'))
            os.chmod(constants.last_upload_results_file, 0o644)
            write_to_disk(constants.lastupload_file)
            os.chmod(constants.lastupload_file, 0o644)

            msg_name = determine_hostname(config.display_name)
            account_number = config.account_number
            if account_number:
                logger.info("Successfully uploaded report from %s to account %s.",
                            msg_name, account_number)
            else:
                logger.info("Successfully uploaded report for %s.", msg_name)
            if config.register:
                # direct to console after register + upload
                logger.info('View the Red Hat Insights console at https://console.redhat.com/insights/')
            break

        elif upload.status_code in (412, 413):
            pconn.handle_fail_rcs(upload)
            raise RuntimeError('Upload failed.')
        else:
            display_upload_error_and_retry(config, tries, "%s: %s" % (upload.status_code, upload.reason))
    return api_response


def upload(config, pconn, tar_file, content_type, collection_duration=None):
    if config.legacy_upload:
        return _legacy_upload(config, pconn, tar_file, content_type, collection_duration)
    logger.info('Uploading Insights data.')
    for tries in range(config.retries):
        logger.debug("Upload attempt %d of %d ...", tries + 1, config.retries)
        try:
            upload = pconn.upload_archive(tar_file, content_type, collection_duration)
        except Exception as e:
            display_upload_error_and_retry(config, tries, str(e))
            continue

        if upload.status_code in (200, 202):
            write_to_disk(constants.lastupload_file)
            os.chmod(constants.lastupload_file, 0o644)
            msg_name = determine_hostname(config.display_name)
            logger.info("Successfully uploaded report for %s.", msg_name)
            if config.register:
                # direct to console after register + upload
                logger.info('View the Red Hat Insights console at https://console.redhat.com/insights/')
            return
        elif upload.status_code in (413, 415):
            pconn.handle_fail_rcs(upload)
            raise RuntimeError('Upload failed.')
        else:
            err_msg = "%s" % upload.status_code
            if hasattr(upload, 'reason'):
                err_msg += ": %s" % upload.reason
            display_upload_error_and_retry(config, tries, err_msg)


def display_upload_error_and_retry(config, tries, error_message):
    logger.error("Upload attempt %d of %d failed! Reason: %s",
                 tries + 1, config.retries, error_message)
    if tries + 1 < config.retries:
        logger.info("Waiting %d seconds then retrying",
                    constants.sleep_time)
        time.sleep(constants.sleep_time)
    else:
        logger.error("All attempts to upload have failed!")
        print("Please see %s for additional information" % config.logging_file)
        raise RuntimeError('Upload failed.')



from insights.parsers.sysconfig import MongodSysconfig
from insights.tests import context_wrap

SYSCONFIG_MONGOD = """
OPTIONS="--quiet -f /etc/mongod.conf"
OPTIONS_EMPTY=""
"""


def test_sysconfig_mongod():
    context = context_wrap(SYSCONFIG_MONGOD, 'etc/sysconfig/mongod')
    sysconf = MongodSysconfig(context)

    assert 'OPTIONS' in sysconf
    assert sysconf.get('OPTIONS') == '--quiet -f /etc/mongod.conf'
    assert sysconf.get('OPTIONS_EMPTY') == ''
    assert sysconf.get('not_exist') is None



# -*- coding: UTF-8 -*-
import os
import six

from insights.specs.datasources.compliance import ComplianceClient
from mock.mock import patch, Mock, mock_open
from pytest import raises

PATH = '/usr/share/xml/scap/ref_id.xml'


@patch("insights.client.config.InsightsConfig", base_url='localhost/app', systemid='', proxy=None)
def test_get_profiles(config):
    compliance_client = ComplianceClient(config=config)
    compliance_client.inventory_id = '068040f1-08c8-43e4-949f-7d6470e9111c'
    compliance_client.conn.session.get = Mock(return_value=Mock(status_code=200, json=Mock(return_value={'data': [{'attributes': 'data'}]})))
    assert compliance_client.get_profiles('search string') == [{'attributes': 'data'}]
    compliance_client.conn.session.get.assert_called_with('https://localhost/app/compliance/profiles', params={'search': 'search string', 'relationships': 'false'})


@patch("insights.client.config.InsightsConfig", base_url='localhost/app', systemid='', proxy=None)
def test_get_profiles_no_profiles(config):
    compliance_client = ComplianceClient(config=config)
    compliance_client.inventory_id = '068040f1-08c8-43e4-949f-7d6470e9111c'
    compliance_client.conn.session.get = Mock(return_value=Mock(status_code=200, json=Mock(return_value={'data': []})))
    assert compliance_client.get_profiles('search string') == []
    compliance_client.conn.session.get.assert_called_with('https://localhost/app/compliance/profiles', params={'search': 'search string', 'relationships': 'false'})


@patch("insights.client.config.InsightsConfig", base_url='localhost/app', systemid='', proxy=None)
def test_get_profiles_error(config):
    compliance_client = ComplianceClient(config=config)
    compliance_client.inventory_id = '068040f1-08c8-43e4-949f-7d6470e9111c'
    compliance_client.conn.session.get = Mock(return_value=Mock(status_code=500))
    assert compliance_client.get_profiles('search string') == []
    compliance_client.conn.session.get.assert_called_with('https://localhost/app/compliance/profiles', params={'search': 'search string', 'relationships': 'false'})


@patch("insights.client.config.InsightsConfig", base_url='localhost/app', systemid='', proxy=None)
def test_get_initial_profiles(config):
    compliance_client = ComplianceClient(config=config)
    compliance_client.inventory_id = '068040f1-08c8-43e4-949f-7d6470e9111c'
    compliance_client.conn.session.get = Mock(return_value=Mock(status_code=200, json=Mock(return_value={'data': [{'attributes': 'data'}]})))
    assert compliance_client.get_initial_profiles() == [{'attributes': 'data'}]
    compliance_client.conn.session.get.assert_called_with('https://localhost/app/compliance/profiles', params={'search': 'system_ids=068040f1-08c8-43e4-949f-7d6470e9111c canonical=false external=false', 'relationships': 'false'})


@patch("insights.client.config.InsightsConfig", base_url='localhost/app', systemid='', proxy=None)
def test_get_profiles_matching_os(config):
    compliance_client = ComplianceClient(os_version=['6', '5'], config=config)
    compliance_client.inventory_id = '068040f1-08c8-43e4-949f-7d6470e9111c'
    compliance_client.conn.session.get = Mock(return_value=Mock(status_code=200, json=Mock(return_value={'data': [{'attributes': 'data'}]})))
    assert compliance_client.get_profiles_matching_os() == [{'attributes': 'data'}]
    compliance_client.conn.session.get.assert_called_with('https://localhost/app/compliance/profiles', params={'search': 'system_ids=068040f1-08c8-43e4-949f-7d6470e9111c canonical=false os_minor_version=5', 'relationships': 'false'})


@patch("insights.client.config.InsightsConfig", base_url='localhost.com/app')
def test_profile_files(config):
    compliance_client = ComplianceClient(config=config)
    compliance_client.os_release = lambda: '7'
    assert compliance_client.profile_files() == []


@patch("insights.specs.datasources.compliance.call", return_value=(0, PATH))
@patch("insights.client.config.InsightsConfig", base_url='localhost.com/app')
def test_find_scap_policy(config, call):
    compliance_client = ComplianceClient(config=config)
    compliance_client.profile_files = lambda: ['/something']
    assert compliance_client.find_scap_policy('ref_id') == PATH


@patch("insights.client.config.InsightsConfig", base_url='localhost.com/app')
def test_find_scap_policy_with_one_datastream_file(config, tmpdir):
    compliance_client = ComplianceClient(config=config)
    dir1 = tmpdir.mkdir('scap')
    file = dir1.join('test_file.xml')
    file.write("""
    <xccdf-1.2:Profile id="xccdf_org.ssgproject.content_profile_anssi_bp28_high">
    </xccdf-1.2:Profile>
        """)
    compliance_client.profile_files = lambda: [str(file)]
    with patch("insights.specs.datasources.compliance.SCAP_DATASTREAMS_PATH", str(dir1) + "/"):
        assert compliance_client.find_scap_policy('content_profile_anssi_bp28_high') == file


@patch("insights.client.config.InsightsConfig", base_url='localhost.com/app')
def test_find_scap_policy_with_two_datastream_file(config, tmpdir):
    compliance_client = ComplianceClient(config=config)
    dir1 = tmpdir.mkdir('scap')
    file1 = dir1.join('test_file1.xml')
    file1.write("""
    <xccdf-1.2:Profile id="xccdf_org.ssgproject.content_profile_anssi_bp28_high">
    </xccdf-1.2:Profile>
        """)
    file2 = dir1.join('test_file2.xml')
    file2.write("""
    <xccdf-1.2:Profile id="xccdf_org.ssgproject.content_profile_anssi_bp28_high">
    </xccdf-1.2:Profile>
        """)
    compliance_client.profile_files = lambda: [str(file1), str(file2)]
    with patch("insights.specs.datasources.compliance.SCAP_DATASTREAMS_PATH", str(dir1) + "/"):
        assert compliance_client.find_scap_policy('content_profile_anssi_bp28_high') == file1


@patch("insights.specs.datasources.compliance.call", return_value=(1, 'bad things happened'.encode('utf-8')))
@patch("insights.client.config.InsightsConfig", base_url='localhost.com/app')
def test_find_scap_policy_not_found(config, call):
    compliance_client = ComplianceClient(config=config)
    compliance_client.profile_files = lambda: ['/something']
    assert compliance_client.find_scap_policy('ref_id') is None


@patch("insights.specs.datasources.compliance.call", return_value=(0, ''.encode('utf-8')))
@patch("insights.client.config.InsightsConfig", base_url='localhost.com/app')
def test_run_scan(config, call):
    compliance_client = ComplianceClient(config=config)
    output_path = '/tmp/oscap_results-ref_id.xml'
    env = os.environ
    env.update({'TZ': 'UTC'})
    compliance_client.run_scan('ref_id', '/nonexistent', output_path)
    if six.PY3:
        call.assert_called_with(("oscap xccdf eval --profile ref_id --results " + output_path + ' /nonexistent'), keep_rc=True, env=env)
    else:
        call.assert_called_with(("oscap xccdf eval --profile ref_id --results " + output_path + ' /nonexistent').encode(), keep_rc=True, env=env)


@patch("insights.specs.datasources.compliance.call", return_value=(1, 'bad things happened'.encode('utf-8')))
@patch("insights.client.config.InsightsConfig", base_url='localhost.com/app')
def test_run_scan_fail(config, call):
    compliance_client = ComplianceClient(config=config)
    output_path = '/tmp/oscap_results-ref_id.xml'
    env = os.environ
    env.update({'TZ': 'UTC'})
    with raises(SystemExit):
        compliance_client.run_scan('ref_id', '/nonexistent', output_path)
    if six.PY3:
        call.assert_called_with(("oscap xccdf eval --profile ref_id --results " + output_path + ' /nonexistent'), keep_rc=True, env=env)
    else:
        call.assert_called_with(("oscap xccdf eval --profile ref_id --results " + output_path + ' /nonexistent').encode(), keep_rc=True, env=env)


@patch("insights.specs.datasources.compliance.call", return_value=(0, ''.encode('utf-8')))
@patch("insights.client.config.InsightsConfig", base_url='localhost.com/app')
def test_run_scan_missing_profile(config, call):
    compliance_client = ComplianceClient(config=config)
    output_path = '/tmp/oscap_results-ref_id.xml'
    env = os.environ
    env.update({'TZ': 'UTC'})
    assert compliance_client.run_scan('ref_id', None, output_path) is None
    call.assert_not_called()


@patch("insights.client.config.InsightsConfig", base_url='localhost.com/app')
def test_tailored_file_is_not_downloaded_if_not_needed(config):
    compliance_client = ComplianceClient(config=config)
    assert compliance_client.download_tailoring_file({'attributes': {'tailored': False}}) is None


@patch("insights.client.config.InsightsConfig", base_url='localhost.com/app')
def test_tailored_file_is_not_downloaded_if_tailored_is_missing(config):
    compliance_client = ComplianceClient(config=config)
    assert compliance_client.download_tailoring_file({'id': 'foo', 'attributes': {'ref_id': 'aaaaa'}}) is None


@patch("insights.specs.datasources.compliance.open", new_callable=mock_open)
@patch("insights.client.config.InsightsConfig", base_url='localhost.com/app')
def test_tailored_file_is_downloaded_from_initial_profile_if_os_minor_version_is_missing(config, call):
    compliance_client = ComplianceClient(config=config)
    compliance_client.conn.session.get = Mock(return_value=Mock(status_code=200, headers={"Content-Type": "application/xml"}, json=Mock(return_value={'data': [{'attributes': 'data'}]})))
    assert 'oscap_tailoring_file-aaaaa' in compliance_client.download_tailoring_file({'id': 'foo', 'attributes': {'tailored': True, 'ref_id': 'aaaaa'}})
    assert compliance_client.download_tailoring_file({'id': 'foo', 'attributes': {'tailored': False, 'ref_id': 'aaaaa'}}) is None


@patch("insights.client.config.InsightsConfig", base_url='localhost.com/app')
def test_tailored_file_is_not_downloaded_if_os_minor_version_mismatches(config):
    compliance_client = ComplianceClient(os_version=['6', '5'], config=config)
    compliance_client.conn.session.get = Mock(return_value=Mock(status_code=200, headers={"Content-Type": "application/xml"}, json=Mock(return_value={'data': [{'attributes': 'data'}]})))
    assert compliance_client.download_tailoring_file({'id': 'foo', 'attributes': {'tailored': True, 'ref_id': 'aaaaa', 'os_minor_version': '2'}}) is None
    assert compliance_client.download_tailoring_file({'id': 'foo', 'attributes': {'tailored': False, 'ref_id': 'aaaaa', 'os_minor_version': '2'}}) is None


@patch("insights.specs.datasources.compliance.open", new_callable=mock_open)
@patch("insights.client.config.InsightsConfig", base_url='localhost.com/app')
def test_tailored_file_is_downloaded_if_needed(config, call):
    compliance_client = ComplianceClient(os_version=['6', '5'], config=config)
    compliance_client.conn.session.get = Mock(return_value=Mock(status_code=200, headers={"Content-Type": "application/xml"}, json=Mock(return_value={'data': [{'attributes': 'data'}]})))
    assert 'oscap_tailoring_file-aaaaa' in compliance_client.download_tailoring_file({'id': 'foo', 'attributes': {'tailored': True, 'ref_id': 'aaaaa', 'os_minor_version': '5'}})
    assert compliance_client.download_tailoring_file({'id': 'foo', 'attributes': {'tailored': False, 'ref_id': 'aaaaa', 'os_minor_version': '5'}}) is None


@patch("insights.specs.datasources.compliance.open", new_callable=mock_open)
@patch("insights.client.config.InsightsConfig", base_url='localhost.com/app')
def test_tailored_file_fails_to_download(config, call):
    compliance_client = ComplianceClient(config=config)

    compliance_client.conn.session.get = Mock(return_value=Mock(status_code=403, ok=False, headers={"Content-Type": "application/xml"}, json=Mock(return_value={'data': [{'attributes': 'data'}]})))
    assert compliance_client.download_tailoring_file({'id': 'foo', 'attributes': {'tailored': True, 'ref_id': 'aaaaa'}}) is None

    compliance_client.conn.session.get = Mock(return_value=Mock(status_code=200, headers={"Content-Type": "application/json"}, json=Mock(return_value={'data': [{'attributes': 'data'}]})))
    assert compliance_client.download_tailoring_file({'id': 'foo', 'attributes': {'tailored': True, 'ref_id': 'aaaaa'}}) is None

    compliance_client.conn.session.get = Mock(return_value=Mock(status_code=200, content=None, headers={"Content-Type": "application/xml"}, json=Mock(return_value={'data': [{'attributes': 'data'}]})))
    assert compliance_client.download_tailoring_file({'id': 'foo', 'attributes': {'tailored': True, 'ref_id': 'aaaaa'}}) is None


@patch("insights.client.config.InsightsConfig", base_url='localhost.com/app')
def test_build_oscap_command_does_not_append_tailoring_path(config):
    compliance_client = ComplianceClient(config=config)
    expected_command = 'oscap xccdf eval --profile aaaaa --results output_path xml_sample'
    assert expected_command == compliance_client.build_oscap_command('aaaaa', 'xml_sample', 'output_path', None)


@patch("insights.client.config.InsightsConfig", base_url='localhost.com/app')
def test_build_oscap_command_append_tailoring_path(config):
    compliance_client = ComplianceClient(config=config)
    expected_command = 'oscap xccdf eval --profile aaaaa --tailoring-file tailoring_path --results output_path xml_sample'
    assert expected_command == compliance_client.build_oscap_command('aaaaa', 'xml_sample', 'output_path', 'tailoring_path')


@patch("insights.client.config.InsightsConfig", base_url='localhost.com/app')
def test__get_inventory_id(config):
    compliance_client = ComplianceClient(config=config)
    compliance_client.conn._fetch_system_by_machine_id = lambda: []
    with raises(SystemExit):
        compliance_client._get_inventory_id()

    compliance_client.conn._fetch_system_by_machine_id = lambda: [{}]
    with raises(SystemExit):
        compliance_client._get_inventory_id()

    compliance_client.conn._fetch_system_by_machine_id = lambda: [{'id': '12345'}]
    assert compliance_client._get_inventory_id() == '12345'



import doctest
import insights.parsers.octavia as octavia_module
from insights.parsers.octavia import OctaviaConf, VALID_KEYS
from insights.tests import context_wrap

CONF_FILE = """
[DEFAULT]
# Print debugging output (set logging level to DEBUG instead of default WARNING level).
debug = False

# Plugin options are hot_plug_plugin (Hot-pluggable controller plugin)
octavia_plugins = hot_plug_plugin

# Hostname to be used by the host machine for services running on it.
# The default value is the hostname of the host machine.
host = some_hostname.some_domain.com

# AMQP Transport URL
# For Single Host, specify one full transport URL:
#   transport_url = rabbit://<user>:<pass>@127.0.0.1:5672/<vhost>
# For HA, specify queue nodes in cluster, comma delimited:
#   transport_url = rabbit://<user>:<pass>@server01,<user>:<pass>@server02/<vhost>
transport_url =

# How long in seconds to wait for octavia worker to exit before killing them.
graceful_shutdown_timeout = 60

log_file = some_file
log_dir = some_dir
policy_file = some_policy_file

[api_settings]
bind_host = 127.0.0.1
bind_port = 9876

# How should authentication be handled (keystone, noauth)
auth_strategy = keystone

allow_pagination = True
allow_sorting = True
pagination_max_limit = 1000
# Base URI for the API for use in pagination links.
# This will be autodetected from the request if not overridden here.
# Example:
#   api_base_uri = http://localhost:9876
api_base_uri = http://localhost:9876

# Enable/disable ability for users to create TLS Terminated listeners
allow_tls_terminated_listeners = True

# Enable/disable ability for users to create PING type Health Monitors
allow_ping_health_monitors = True

# Dictionary of enabled provider driver names and descriptions
# A comma separated list of dictionaries of the enabled provider driver names
# and descriptions.
enabled_provider_drivers = amphora:The Octavia Amphora driver.,octavia: \\
                           Deprecated alias of the Octavia Amphora driver.

# Default provider driver
default_provider_driver = amphora

# The minimum health monitor delay interval for UDP-CONNECT Health Monitor type
udp_connect_min_interval_health_monitor = 3

[database]
# This line MUST be changed to actually run the plugin.
# Example:
# connection = mysql+pymysql://root:pass@127.0.0.1:3306/octavia
# Replace 127.0.0.1 above with the IP address of the database used by the
# main octavia server. (Leave it as is if the database runs on this host.)

connection = mysql+pymysql://

# NOTE: In deployment the [database] section and its connection attribute may
# be set in the corresponding core plugin '.ini' file. However, it is suggested
# to put the [database] section and its connection attribute in this
# configuration file.

[health_manager]
bind_ip = 127.0.0.1
bind_port = 5555
# controller_ip_port_list example: 127.0.0.1:5555, 127.0.0.1:5555
controller_ip_port_list = 127.0.0.1:5555, 127.0.0.1:5555
failover_threads = 10
# status_update_threads will default to the number of processors on the host.
# This setting is deprecated and if you specify health_update_threads and
# stats_update_threads, they override this parameter.
status_update_threads = 10
# health_update_threads will default to the number of processors on the host
health_update_threads = 10
# stats_update_threads will default to the number of processors on the host
stats_update_threads = 10
heartbeat_interval = 10
# Symmetric encrpytion key
heartbeat_key =
heartbeat_timeout = 60
health_check_interval = 3
sock_rlimit = 0

# Health/StatsUpdate options are
#                           *_db
#                           *_logger
health_update_driver = health_db
stats_update_driver = stats_db

[keystone_authtoken]
# This group of config options are imported from keystone middleware. Thus the
# option names should match the names declared in the middleware.
# The www_authenticate_uri is the public endpoint and is returned in headers on a 401
# www_authenticate_uri = https://localhost:5000/v3
# The auth_url is the admin endpoint actually used for validating tokens
auth_url = https://localhost:5000/v3
username = octavia
password = password
project_name = service

# Domain names must be set, these are *not* default but work for most clouds
# project_domain_name = Default
user_domain_name = Default

insecure = False
cafile =

[certificates]
# Certificate Generator options are local_cert_generator
cert_generator = local_cert_generator

# For local certificate signing:
ca_certificate = /etc/ssl/certs/ssl-cert-snakeoil.pem
ca_private_key = /etc/ssl/private/ssl-cert-snakeoil.key
ca_private_key_passphrase =
server_certs_key_passphrase = do-not-use-this-key
signing_digest = sha256
cert_validity_time = 2592000  # 30 days = 30d * 24h * 60m * 60s = 2592000s
storage_path = /var/lib/octavia/certificates/

# For the TLS management
# Certificate Manager options are local_cert_manager
#                                 barbican_cert_manager
#                                 castellan_cert_manager
cert_manager = barbican_cert_manager
# For Barbican authentication (if using any Barbican based cert class)
barbican_auth = barbican_acl_auth
#
# Region in Identity service catalog to use for communication with the Barbican service.
region_name = some_region
#
# Endpoint type to use for communication with the Barbican service.
endpoint_type = publicURL

[networking]
# The maximum attempts to retry an action with the networking service.
max_retries = 15
# Seconds to wait before retrying an action with the networking service.
retry_interval = 1
# The maximum time to wait, in seconds, for a port to detach from an amphora
port_detach_timeout = 300
# Allow/disallow specific network object types when creating VIPs.
allow_vip_network_id = True
allow_vip_subnet_id = True
allow_vip_port_id = True
# List of network_ids that are valid for VIP creation.
# If this field empty, no validation is performed.
valid_vip_networks =
# List of reserved IP addresses that cannot be used for member addresses
# The default is the nova metadata service address
reserved_ips = ['169.254.169.254']

[haproxy_amphora]
base_path = /var/lib/octavia
base_cert_dir = /var/lib/octavia/certs
# Absolute path to a custom HAProxy template file
haproxy_template = /some/path
connection_logging = True
connection_max_retries = 120
connection_retry_interval = 5
build_rate_limit = -1
build_active_retries = 120
build_retry_interval = 5

# Maximum number of entries that can fit in the stick table.
# The size supports "k", "m", "g" suffixes.
haproxy_stick_size = 10k

# REST Driver specific
bind_host = 0.0.0.0
bind_port = 9443
#
# This setting is only needed with IPv6 link-local addresses (fe80::/64) are
# used for communication between Octavia and its Amphora, if IPv4 or other IPv6
# addresses are used it can be ignored.
lb_network_interface = o-hm0
#
haproxy_cmd = /usr/sbin/haproxy
respawn_count = 2
respawn_interval = 2
client_cert = /etc/octavia/certs/client.pem
server_ca = /etc/octavia/certs/server_ca.pem
#
# This setting is deprecated. It is now automatically discovered.
use_upstart = True
#
rest_request_conn_timeout = 10
rest_request_read_timeout = 60
#
# These "active" timeouts are used once the amphora should already
# be fully up and active. These values are lower than the other values to
# facilitate "fail fast" scenarios like failovers
active_connection_max_retries = 15
active_connection_rety_interval = 2

# The user flow log format for HAProxy.
# {{ project_id }} and {{ lb_id }} will be automatically substituted by the
# controller when configuring HAProxy if they are present in the string.
user_log_format = '{{ project_id }} {{ lb_id }} %f %ci %cp %t %{+Q}r %ST %B %U %[ssl_c_verify] %{+Q}[ssl_c_s_dn] %b %s %Tt %tsc'

[controller_worker]
workers = 1
amp_active_retries = 30
amp_active_wait_sec = 10
# Glance parameters to extract image ID to use for amphora. Only one of
# parameters is needed. Using tags is the recommended way to refer to images.
amp_image_id =
amp_image_tag =
# Optional owner ID used to restrict glance images to one owner ID.
# This is a recommended security setting.
amp_image_owner_id =
# Nova parameters to use when booting amphora
amp_flavor_id =
# Upload the ssh key as the service_auth user described elsewhere in this config.
# Leaving this variable blank will install no ssh key on the amphora.
amp_ssh_key_name =

amp_ssh_access_allowed = True

# Networks to attach to the Amphorae examples:
#  - One primary network
#  - - amp_boot_network_list = 22222222-3333-4444-5555-666666666666
#  - Multiple networks
#  - - amp_boot_network_list = 11111111-2222-33333-4444-555555555555, 22222222-3333-4444-5555-666666666666
#  - All networks defined in the list will be attached to each amphora
amp_boot_network_list =

amp_secgroup_list =
client_ca = /etc/octavia/certs/ca_01.pem

# Amphora driver options are amphora_noop_driver,
#                            amphora_haproxy_rest_driver
#
amphora_driver = amphora_noop_driver
#
# Compute driver options are compute_noop_driver
#                            compute_nova_driver
#
compute_driver = compute_noop_driver
#
# Network driver options are network_noop_driver
#                            allowed_address_pairs_driver
#
network_driver = network_noop_driver
# Volume driver options are volume_noop_driver
#                           volume_cinder_driver
#
volume_driver = volume_noop_driver
#
# Distributor driver options are distributor_noop_driver
#                                single_VIP_amphora
#
distributor_driver = distributor_noop_driver
#
# Load balancer topology options are SINGLE, ACTIVE_STANDBY
loadbalancer_topology = SINGLE
user_data_config_drive = False

[task_flow]
# TaskFlow engine options are:
#   - serial:   Runs all tasks on a single thread.
#   - parallel: Schedules tasks onto different threads to allow
#               for running non-dependent tasks simultaneously
#
engine = parallel
max_workers = 5
#
# This setting prevents the controller worker from reverting taskflow flows.
# This will leave resources in an inconsistent state and should only be used
# for debugging purposes.
disable_revert = False

[oslo_messaging]
# Queue Consumer Thread Pool Size
rpc_thread_pool_size = 2

# Topic (i.e. Queue) Name
topic = octavia_prov

[oslo_middleware]
# HTTPProxyToWSGI middleware enabled
enable_proxy_headers_parsing = False

[house_keeping]
# Interval in seconds to initiate spare amphora checks
spare_check_interval = 30
spare_amphora_pool_size = 0

# Cleanup interval for Deleted amphora
cleanup_interval = 30
# Amphora expiry age in seconds. Default is 1 week
amphora_expiry_age = 604800

# Load balancer expiry age in seconds. Default is 1 week
load_balancer_expiry_age = 604800

[amphora_agent]
agent_server_ca = /etc/octavia/certs/client_ca.pem
agent_server_cert = /etc/octavia/certs/server.pem

# Defaults for agent_server_network_dir when not specified here are:
# Ubuntu: /etc/netns/amphora-haproxy/network/interfaces.d/
# Centos/fedora/rhel: /etc/netns/amphora-haproxy/sysconfig/network-scripts/
#
agent_server_network_dir =

agent_server_network_file =
agent_request_read_timeout = 180

# Minimum TLS protocol, eg: TLS, TLSv1.1, TLSv1.2, TLSv1.3 (if available)
agent_tls_protocol = TLSv1.2

# Amphora default UDP driver is keepalived_lvs
#
amphora_udp_driver = keepalived_lvs

##### Log offloading
#
# Note: The admin and tenant logs can point to the same endpoints.
#
# List of log server ip and port pairs for Administrative logs.
# Additional hosts are backup to the primary server. If none are
# specified, remote logging is disabled.
# Example 192.0.2.1:10514, 2001:db8:1::10:10514'
#
admin_log_targets =
#
# List of log server ip and port pairs for tenant traffic logs.
# Additional hosts are backup to the primary server. If none are
# specified, remote logging is disabled.
# Example 192.0.2.1:10514, 2001:db8:2::15:10514'
#
tenant_log_targets =

# Sets the syslog LOG_LOCAL[0-7] facility number for amphora log offloading.
# user_log_facility will receive the traffic flow logs.
# administrative_log_facility will receive the amphora processes logs.
# Note: Some processes only support LOG_LOCAL, so we are restricted to the
#       LOG_LOCAL facilities.
#
user_log_facility = 0
administrative_log_facility = 1

# The log forwarding protocol to use. One of TCP or UDP.
log_protocol = UDP

# The maximum attempts to retry connecting to the logging host.
log_retry_count = 5

# The time, in seconds, to wait between retries connecting to the logging host.
log_retry_interval = 2

# The queue size (messages) to buffer log messages.
log_queue_size = 10000

# Controller local path to a custom logging configuration template.
# Currently this is an rsyslog configuration file template.
logging_template_override =

# When True, the amphora will forward all of the system logs (except tenant
# traffice logs) to the admin log target(s). When False, only amphora specific
# admin logs will be forwarded.
forward_all_logs = False

# When True, no logs will be written to the amphora filesystem. When False,
# log files will be written to the local filesystem.
disable_local_log_storage = False

[keepalived_vrrp]
# Amphora Role/Priority advertisement interval in seconds
vrrp_advert_int = 1

# Service health check interval and success/fail count
vrrp_check_interval = 5
vrrp_fail_count = 2
vrrp_success_count = 2

# Amphora MASTER gratuitous ARP refresh settings
vrrp_garp_refresh_interval = 5
vrrp_garp_refresh_count = 2

[service_auth]
memcached_servers =
cafile = /opt/stack/data/ca-bundle.pem
project_domain_name = Default
project_name = admin
user_domain_name = Default
password = password
username = admin
auth_type = password
auth_url = http://localhost:5555/

[nova]
# The name of the nova service in the keystone catalog
service_name =
# Custom nova endpoint if override is necessary
endpoint =

# Region in Identity service catalog to use for communication with the
# OpenStack services.
region_name =

# Endpoint type in Identity service catalog to use for communication with
# the OpenStack services.
endpoint_type = publicURL

# CA certificates file to verify neutron connections when TLS is enabled
ca_certificates_file =

# Disable certificate validation on SSL connections
insecure = False

# If non-zero, generate a random name of the length provided for each amphora,
# in the format "a[A-Z0-9]*".
# Otherwise, the default name format will be used: "amphora-{UUID}".
random_amphora_name_length = 0
#
# Availability zone to use for creating Amphorae
availability_zone =

# Enable anti-affinity in nova
enable_anti_affinity = False
# Set the anti-affinity policy to what is suitable.
# Nova supports: anti-affinity and soft-anti-affinity
anti_affinity_policy = anti-affinity

[cinder]
# The name of the cinder service in the keystone catalog
service_name =
# Custom cinder endpoint if override is necessary
endpoint =

# Region in Identity service catalog to use for communication with the
# OpenStack services.
region_name =

# Endpoint type in Identity service catalog to use for communication with
# the OpenStack services.
endpoint_type = publicURL

# Availability zone to use for creating Volume
availability_zone =

# CA certificates file to verify cinder connections when TLS is enabled
insecure = False
ca_certificates_file =

# Size of root volume in GB for Amphora Instance when use Cinder
# In some storage backends such as ScaleIO, the size of volume is multiple of 8
volume_size = 16

# Volume type to be used for Amphora Instance root disk
# If not specified, default_volume_type from cinder.conf will be used
volume_type =

# Interval time to wait until volume becomes available
volume_create_retry_interval = 5

# Timeout to wait for volume creation success
volume_create_timeout = 300

# Maximum number of retries to create volume
volume_create_max_retries = 5

[glance]
# The name of the glance service in the keystone catalog
service_name =
# Custom glance endpoint if override is necessary
endpoint =

# Region in Identity service catalog to use for communication with the
# OpenStack services.
region_name =

# Endpoint type in Identity service catalog to use for communication with
# the OpenStack services.
endpoint_type = publicURL

# CA certificates file to verify neutron connections when TLS is enabled
insecure = False
ca_certificates_file =

[neutron]
# The name of the neutron service in the keystone catalog
service_name =
# Custom neutron endpoint if override is necessary
endpoint =

# Region in Identity service catalog to use for communication with the
# OpenStack services.
region_name =

# Endpoint type in Identity service catalog to use for communication with
# the OpenStack services.
endpoint_type = publicURL

# CA certificates file to verify neutron connections when TLS is enabled
insecure = False
ca_certificates_file =

[quotas]
default_load_balancer_quota = -1
default_listener_quota = -1
default_member_quota = -1
default_pool_quota = -1
default_health_monitor_quota = -1

[audit]
# Enable auditing of API requests.
enabled = False

# Path to audit map file for octavia-api service. Used only
# when API audit is enabled.
audit_map_file = /etc/octavia/octavia_api_audit_map.conf

# Comma separated list of REST API HTTP methods to be
# ignored during audit. For example: auditing will not be done
# on any GET or POST requests if this is set to "GET,POST". It
# is used only when API audit is enabled.
ignore_req_list =

[audit_middleware_notifications]
# Note: This section comes from openstack/keystonemiddleware
# It is included here for documentation convenience and may be out of date

# Indicate whether to use oslo_messaging as the notifier. If set to False,
# the local logger will be used as the notifier. If set to True, the
# oslo_messaging package must also be present. Otherwise, the local will be
# used instead.
use_oslo_messaging = True

# The Driver to handle sending notifications. Possible values are messaging,
# messagingv2, routing, log, test, noop. If not specified, then value from
# oslo_messaging_notifications conf section is used.
driver =

# List of AMQP topics used for OpenStack notifications. If not specified,
# then value from oslo_messaging_notifications conf section is used.
topics =

# A URL representing messaging driver to use for notification. If not
# specified, we fall back to the same configuration used for RPC.
transport_url =

[driver_agent]
status_socket_path = /var/run/octavia/status.sock
stats_socket_path = /var/run/octavia/stats.sock
get_socket_path = /var/run/octavia/get.sock

# Maximum time to wait for a status message before checking for shutdown
status_request_timeout = 5

# Maximum number of status processes per driver-agent
status_max_processes = 50

# Maximum time to wait for a stats message before checking for shutdown
stats_request_timeout = 5

# Maximum number of stats processes per driver-agent
stats_max_processes = 50

# Percentage of max_processes (both status and stats) in use to start
# logging warning messages about an overloaded driver-agent.
max_process_warning_percent = .75

# How long in seconds to wait for provider agents to exit before killing them.
provider_agent_shutdown_timeout = 60

# List of enabled provider agents.
enabled_provider_agents =
"""

DEFAULT_OPTIONS = set([
    'debug', 'octavia_plugins', 'graceful_shutdown_timeout',
    'log_file', 'log_dir', 'policy_file'
])


def test_full_conf():
    # Simulate filtering to allow testing filtered data
    filtered_content = []
    for line in CONF_FILE.strip().splitlines():
        if any([f in line for f in VALID_KEYS]):
            filtered_content.append(line)
    octavia_conf = OctaviaConf(context_wrap('\n'.join(filtered_content)))
    assert octavia_conf is not None
    assert set(octavia_conf.defaults().keys()) == DEFAULT_OPTIONS
    assert octavia_conf.defaults()['debug'] == 'False'
    assert octavia_conf.defaults()['octavia_plugins'] == 'hot_plug_plugin'

    assert 'api_settings' in octavia_conf
    assert set(octavia_conf.items('api_settings').keys()) == set([
        'bind_host', 'bind_port', 'auth_strategy', 'allow_pagination', 'allow_sorting',
        'pagination_max_limit', 'api_base_uri', 'allow_tls_terminated_listeners',
        'allow_ping_health_monitors', 'enabled_provider_drivers', 'default_provider_driver',
        'udp_connect_min_interval_health_monitor'
    ]) | DEFAULT_OPTIONS

    assert 'database' in octavia_conf
    assert set(octavia_conf.items('database').keys()) == DEFAULT_OPTIONS

    assert 'health_manager' in octavia_conf
    assert set(octavia_conf.items('health_manager').keys()) == set([
        'bind_ip', 'bind_port', 'controller_ip_port_list', 'failover_threads',
        'status_update_threads', 'health_update_threads', 'stats_update_threads',
        'heartbeat_interval', 'heartbeat_timeout', 'health_check_interval',
        'sock_rlimit', 'health_update_driver', 'stats_update_driver'
    ]) | DEFAULT_OPTIONS

    assert 'keystone_authtoken' in octavia_conf
    assert set(octavia_conf.items('keystone_authtoken').keys()) == set(['insecure', 'cafile']) | DEFAULT_OPTIONS

    assert 'certificates' in octavia_conf
    assert set(octavia_conf.items('certificates').keys()) == set([
        'cert_generator', 'signing_digest', 'cert_validity_time', 'storage_path',
        'cert_manager', 'region_name', 'endpoint_type'
    ]) | DEFAULT_OPTIONS

    assert 'networking' in octavia_conf
    assert set(octavia_conf.items('networking').keys()) == set([
        'max_retries', 'retry_interval', 'port_detach_timeout', 'allow_vip_network_id',
        'allow_vip_subnet_id', 'allow_vip_port_id', 'reserved_ips'
    ]) | DEFAULT_OPTIONS

    assert 'haproxy_amphora' in octavia_conf
    assert set(octavia_conf.items('haproxy_amphora').keys()) == set([
        'base_path',
        'base_cert_dir',
        'haproxy_template',
        'connection_logging',
        'connection_max_retries',
        'connection_retry_interval',
        'build_rate_limit',
        'build_active_retries',
        'build_retry_interval',
        'haproxy_stick_size',
        'bind_host',
        'bind_port',
        'lb_network_interface',
        'haproxy_cmd',
        'respawn_count',
        'respawn_interval',
        'client_cert',
        'server_ca',
        'use_upstart',
        'rest_request_conn_timeout',
        'rest_request_read_timeout',
        'active_connection_max_retries',
        'active_connection_rety_interval',
        'user_log_format',
    ]) | DEFAULT_OPTIONS

    assert 'controller_worker' in octavia_conf
    assert set(octavia_conf.items('controller_worker').keys()) == set([
        'workers',
        'amp_active_retries',
        'amp_active_wait_sec',
        'amp_image_id',
        'amp_image_tag',
        'amp_image_owner_id',
        'amp_flavor_id',
        'amp_boot_network_list',
        'amp_secgroup_list',
        'amp_ssh_access_allowed',
        'client_ca',
        'amphora_driver',
        'compute_driver',
        'network_driver',
        'volume_driver',
        'distributor_driver',
        'loadbalancer_topology',
        'user_data_config_drive',
    ]) | DEFAULT_OPTIONS

    assert 'task_flow' in octavia_conf
    assert set(octavia_conf.items('task_flow').keys()) == set([
        'engine',
        'max_workers',
        'disable_revert',
    ]) | DEFAULT_OPTIONS

    assert 'oslo_messaging' in octavia_conf
    assert set(octavia_conf.items('oslo_messaging').keys()) == set([
        'rpc_thread_pool_size',
        'topic',
    ]) | DEFAULT_OPTIONS

    assert 'oslo_middleware' in octavia_conf
    assert set(octavia_conf.items('oslo_middleware').keys()) == set([
        'enable_proxy_headers_parsing',
    ]) | DEFAULT_OPTIONS

    assert 'house_keeping' in octavia_conf
    assert set(octavia_conf.items('house_keeping').keys()) == set([
        'spare_check_interval',
        'spare_amphora_pool_size',
        'cleanup_interval',
        'amphora_expiry_age',
        'load_balancer_expiry_age',
    ]) | DEFAULT_OPTIONS

    assert 'amphora_agent' in octavia_conf
    assert set(octavia_conf.items('amphora_agent').keys()) == set([
        'agent_server_ca',
        'agent_server_cert',
        'agent_server_network_dir',
        'agent_server_network_file',
        'agent_request_read_timeout',
        'agent_tls_protocol',
        'amphora_udp_driver',
        'admin_log_targets',
        'tenant_log_targets',
        'user_log_facility',
        'administrative_log_facility',
        'log_protocol',
        'log_retry_count',
        'log_retry_interval',
        'log_queue_size',
        'logging_template_override',
        'forward_all_logs',
        'disable_local_log_storage',
    ]) | DEFAULT_OPTIONS

    assert 'keepalived_vrrp' in octavia_conf
    assert set(octavia_conf.items('keepalived_vrrp').keys()) == set([
        'vrrp_advert_int',
        'vrrp_check_interval',
        'vrrp_fail_count',
        'vrrp_success_count',
        'vrrp_garp_refresh_interval',
        'vrrp_garp_refresh_count',
    ]) | DEFAULT_OPTIONS

    assert 'service_auth' in octavia_conf
    assert set(octavia_conf.items('service_auth').keys()) == set([
        'memcached_servers',
        'cafile',
        'auth_type',
    ]) | DEFAULT_OPTIONS

    assert 'nova' in octavia_conf
    assert set(octavia_conf.items('nova').keys()) == set([
        'service_name',
        'region_name',
        'endpoint_type',
        'ca_certificates_file',
        'insecure',
        'random_amphora_name_length',
        'availability_zone',
        'enable_anti_affinity',
        'anti_affinity_policy',
    ]) | DEFAULT_OPTIONS

    assert 'cinder' in octavia_conf
    assert set(octavia_conf.items('cinder').keys()) == set([
        'service_name',
        'region_name',
        'endpoint_type',
        'availability_zone',
        'insecure',
        'ca_certificates_file',
        'volume_size',
        'volume_type',
        'volume_create_retry_interval',
        'volume_create_timeout',
        'volume_create_max_retries',
    ]) | DEFAULT_OPTIONS

    assert 'glance' in octavia_conf
    assert set(octavia_conf.items('glance').keys()) == set([
        'service_name',
        'region_name',
        'endpoint_type',
        'insecure',
        'ca_certificates_file',
    ]) | DEFAULT_OPTIONS

    assert 'neutron' in octavia_conf
    assert set(octavia_conf.items('neutron').keys()) == set([
        'service_name',
        'region_name',
        'endpoint_type',
        'insecure',
        'ca_certificates_file',
    ]) | DEFAULT_OPTIONS

    assert 'quotas' in octavia_conf
    assert set(octavia_conf.items('quotas').keys()) == set([
        'default_load_balancer_quota',
        'default_listener_quota',
        'default_member_quota',
        'default_pool_quota',
        'default_health_monitor_quota',
    ]) | DEFAULT_OPTIONS

    assert 'audit' in octavia_conf
    assert set(octavia_conf.items('audit').keys()) == set([
        'enabled',
        'audit_map_file',
        'ignore_req_list',
    ]) | DEFAULT_OPTIONS

    assert 'audit_middleware_notifications' in octavia_conf
    assert set(octavia_conf.items('audit_middleware_notifications').keys()) == set([
        'use_oslo_messaging',
        'driver',
        'topics',
    ]) | DEFAULT_OPTIONS

    assert 'driver_agent' in octavia_conf
    assert set(octavia_conf.items('driver_agent').keys()) == set([
        'status_socket_path',
        'stats_socket_path',
        'get_socket_path',
        'status_request_timeout',
        'status_max_processes',
        'stats_request_timeout',
        'stats_max_processes',
        'max_process_warning_percent',
        'provider_agent_shutdown_timeout',
        'enabled_provider_agents',
    ]) | DEFAULT_OPTIONS


def test_doc_examples():
    env = {
        'octavia_conf': OctaviaConf(context_wrap(CONF_FILE)),
    }
    failed, total = doctest.testmod(octavia_module, globs=env)
    assert failed == 0
